# -*- coding: utf-8 -*-
from django import forms
from django.contrib.admin.widgets import AdminFileWidget
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _


class CenterMapWidget(forms.Widget):

    class Media:
        js = (
            '/static/djeym/js/jquery.min.js',
            '/static/djeym/js/admin_center_map_widget.min.js'
        )

    def render(self, name, value, attrs=None, renderer=None):
        """Render the widget as an HTML string."""
        text_hint = _(
            "Move the marker or click on the map in the right place.")
        html_hint = "<div class=\"hint_center_map\">{}</div>".format(text_hint)
        html_map = "<div id=\"id_center_map\"></div>"
        text_input_html = "{0}{1}".format(html_hint, html_map)
        return mark_safe(text_input_html)


class CheckIconOffsetWidget(forms.Widget):

    class Media:
        js = (
            '/static/djeym/js/jquery.min.js',
            '/static/djeym/js/admin_check_icon_offset_widget.min.js'
        )

    def render(self, name, value, attrs=None, renderer=None):
        """Render the widget as an HTML string."""
        text_hint = _("Changing values along the axes X and Y, "
                      "match your marker with the standard")
        img_hint = "<img src=\"/static/djeym/img/hint.svg\" class=\"hint_offset\" alt=\"hint\">"
        html_hint = "<div class=\"hint_check_icon_offset\">{0} {1}</div>"\
            .format(text_hint, img_hint)
        html_map = "<div id=\"id_check_icon_offset_map\"></div>"
        text_input_html = "{0}{1}".format(html_hint, html_map)
        return mark_safe(text_input_html)


class AdminFileThumbWidget(AdminFileWidget):
    template_name = "djeym/admin/widgets/clearable_file_input.html"



# -*- coding: utf-8 -*-
import os

from django.core.files import File
from django.core.management import BaseCommand, CommandError
from django.db import IntegrityError, transaction
from django.utils.translation import ugettext as _
from slugify import slugify

from djeym.models import IconCollection, MarkerIcon

# --name - 'Icon Collection Name
# --path - '/home/user_name/The path to the directory with icons'
# python manage.py exporticons --name '' --path ''


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('--name', type=str, default="")
        parser.add_argument('--path', type=str, default="")

    def handle(self, *args, **options):
        collection_name = options['name']
        dir_path = options['path']

        if len(collection_name) == 0:
            msg = _('Error - Collection name not specified. Parameter: --name')
            raise CommandError(msg)

        if not os.path.isdir(dir_path):
            msg = _('Error - Specify a directory with icons. Parameter: --path')
            raise CommandError(msg)

        if IconCollection.objects.filter(slug=slugify(collection_name)).count() != 0:
            msg = _('The {} collection already exists.')
            raise CommandError(msg.format(collection_name))

        collection = IconCollection.objects.create(title=collection_name)
        icon_list = os.listdir(path=dir_path)

        try:
            with transaction.atomic():
                for icon in icon_list:
                    icon_path = "{0}/{1}".format(dir_path, icon)
                    with open(icon_path, mode='rb') as svg_file:
                        MarkerIcon.objects.create(
                            icon_collection=collection,
                            title=os.path.splitext(icon)[0],
                            svg=File(svg_file))
        except IntegrityError as err:
            raise CommandError(err)



# Generated by Django 3.0.2 on 2020-01-27 05:58

import adminsortable.fields
import ckeditor_uploader.fields
import colorful.fields
from decimal import Decimal
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import djeym.utils


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='BlockedIP',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ip', models.GenericIPAddressField(null=True, unique=True, verbose_name='IP-address')),
                ('timestamp', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Date and Time')),
            ],
            options={
                'verbose_name': 'Blocked IP',
                'verbose_name_plural': 'Blocked IPs',
            },
        ),
        migrations.CreateModel(
            name='CategoryPlacemark',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='Name of any geographic area - Country Name or City Name or District Name or The name of the resort coast, etc.', max_length=60, verbose_name='Title')),
                ('category_icon', models.CharField(default='', help_text='http://materialdesignicons.com/ - Example: help OR mdi-help', max_length=255, verbose_name='Category Icon')),
                ('category_color', colorful.fields.RGBColorField(colors=('#F44336', '#C62828', '#E91E63', '#AD1457', '#9C27B0', '#6A1B9A', '#673AB7', '#4527A0', '#3F51B5', '#283593', '#2196F3', '#1565C0', '#0091EA', '#03A9F4', '#0277BD', '#00BCD4', '#00838F', '#009688', '#00695C', '#4CAF50', '#2E7D32', '#00C853', '#8BC34A', '#558B2F', '#CDDC39', '#9E9D24', '#FFEB3B', '#F9A825', '#FFC107', '#FF8F00', '#FF9800', '#EF6C00', '#FF5722', '#D84315', '#795548', '#4E342E', '#607D8B', '#37474F', '#FAFAFA', '#F5F5F5', '#EEEEEE', '#E0E0E0', '#BDBDBD', '#9E9E9E', '#757575', '#616161', '#424242', '#212121', '#000000', '#FFFFFF'), default='#00C853', verbose_name='Category color')),
                ('position', models.PositiveIntegerField(db_index=True, default=0, editable=False)),
                ('active', models.BooleanField(default=True, verbose_name='Active category ?')),
            ],
            options={
                'verbose_name': '-Category of marker',
                'verbose_name_plural': '-Category of markers',
                'ordering': ('position',),
            },
        ),
        migrations.CreateModel(
            name='ClusterIcon',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('svg', models.FileField(help_text='Only SVG files.', null=True, upload_to=djeym.utils.make_upload_path, validators=[djeym.utils.validate_svg], verbose_name='Icon')),
                ('title', models.CharField(default='', help_text='Example: Green Garden', max_length=60, unique=True, verbose_name='Cluster name')),
                ('size_width', models.PositiveSmallIntegerField(default=0, verbose_name='Icon width')),
                ('size_height', models.PositiveSmallIntegerField(default=0, verbose_name='Icon height')),
                ('offset_x', models.DecimalField(decimal_places=1, default=Decimal('0.0'), max_digits=3, verbose_name='Offset by axis - X')),
                ('offset_y', models.DecimalField(decimal_places=1, default=Decimal('0.0'), max_digits=3, verbose_name='Offset by axis - Y')),
            ],
            options={
                'verbose_name': 'Icons for clusters',
                'verbose_name_plural': 'Icons for clusters',
                'ordering': ('title', 'id'),
            },
        ),
        migrations.CreateModel(
            name='IconCollection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(default='', help_text='Example: Сherry Light Amber', max_length=60, unique=True, verbose_name='Collection name')),
                ('slug', models.SlugField(blank=True, max_length=255, null=True, unique=True)),
            ],
            options={
                'verbose_name': 'Icon collections for markers',
                'verbose_name_plural': 'Icon collections for markers',
                'ordering': ('title', 'id'),
            },
        ),
        migrations.CreateModel(
            name='LoadIndicator',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('svg', models.FileField(null=True, upload_to=djeym.utils.make_upload_path, validators=[djeym.utils.validate_svg], verbose_name='Icon')),
                ('title', models.CharField(default='', max_length=60, unique=True, verbose_name='Title')),
                ('slug', models.SlugField(blank=True, max_length=255, null=True)),
            ],
            options={
                'verbose_name': 'Load indicator',
                'verbose_name_plural': 'Load indicators',
                'ordering': ('title', 'id'),
            },
        ),
        migrations.CreateModel(
            name='Map',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='Examples: Supermarkets of the city of Kharkov or Hawaii - Oahu Island or Luxury hotels in Honolulu or Parking of primitive people, etc.', max_length=60, unique=True, verbose_name='Title')),
                ('load_indicator_size', models.PositiveSmallIntegerField(choices=[(64, '64'), (96, '96'), (128, '128')], default=64, editable=False, verbose_name='Load indicator size')),
                ('animation_speed', models.CharField(choices=[('0.3', '0.3'), ('0.4', '0.4'), ('0.5', '0.5'), ('0.6', '0.6'), ('0.7', '0.7'), ('0.8', '0.8'), ('0.9', '0.9'), ('1', '1'), ('1.1', '1.1'), ('1.2', '1.2'), ('1.3', '1.3'), ('1.4', '1.4'), ('1.5', '1.5'), ('1.6', '1.6'), ('1.7', '1.7'), ('1.8', '1.8')], default='0.8', editable=False, max_length=255, verbose_name='Load indicator animation speed')),
                ('disable_indicator_animation', models.BooleanField(default=False, editable=False, help_text='It may be useful for the abbreviation or logo of the company, if it does not make sense to animate them.', verbose_name='Disable loading indicator animation')),
                ('slug', models.SlugField(blank=True, max_length=255, null=True, unique=True)),
                ('active', models.BooleanField(default=True, verbose_name='Active map ?')),
                ('zoom', models.PositiveSmallIntegerField(choices=[(0, 0), (1, 1), (2, 2), (3, 3), (4, 4), (5, 5), (6, 6), (7, 7), (8, 8), (9, 9), (10, 10), (11, 11), (12, 12), (13, 13), (14, 14), (15, 15), (16, 16), (17, 17), (18, 18), (19, 19), (20, 20), (21, 21), (22, 22), (23, 23)], default=3, verbose_name='Zoom')),
                ('latitude', models.CharField(default='0', help_text='Latitude center of the map.', max_length=255, validators=[djeym.utils.validate_coordinates], verbose_name='Latitude')),
                ('longitude', models.CharField(default='0', help_text='Longitude center of the map.', max_length=255, validators=[djeym.utils.validate_coordinates], verbose_name='Longitude')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('demo_categories', models.BooleanField(blank=True, default=True, editable=False, verbose_name='Add one-time categories')),
                ('icon_cluster', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='ymap', to='djeym.ClusterIcon', verbose_name='Icon for cluster')),
                ('icon_collection', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='ymap', to='djeym.IconCollection', verbose_name='Icon collection for markers')),
                ('load_indicator', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='ymap', to='djeym.LoadIndicator', verbose_name='Load indicator')),
            ],
            options={
                'verbose_name': 'Map',
                'verbose_name_plural': 'Map',
                'ordering': ('title',),
            },
        ),
        migrations.CreateModel(
            name='Statistics',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('obj_type', models.CharField(default='', max_length=255, verbose_name='Object type')),
                ('obj_id', models.PositiveIntegerField(default=0, verbose_name='Object ID')),
                ('ip', models.GenericIPAddressField(null=True, verbose_name='IP-address')),
                ('likes', models.BooleanField(default=False, verbose_name='Likes')),
                ('timestamp', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Date and Time')),
            ],
            options={
                'verbose_name': 'Record',
                'verbose_name_plural': 'Statistics',
            },
        ),
        migrations.CreateModel(
            name='TileSource',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(default='', max_length=255, unique=True, verbose_name='Title')),
                ('maxzoom', models.PositiveSmallIntegerField(choices=[(0, 0), (1, 1), (2, 2), (3, 3), (4, 4), (5, 5), (6, 6), (7, 7), (8, 8), (9, 9), (10, 10), (11, 11), (12, 12), (13, 13), (14, 14), (15, 15), (16, 16), (17, 17), (18, 18), (19, 19), (20, 20), (21, 21), (22, 22), (23, 23)], default=12, verbose_name='Maximum zoom')),
                ('minzoom', models.PositiveSmallIntegerField(choices=[(0, 0), (1, 1), (2, 2), (3, 3), (4, 4), (5, 5), (6, 6), (7, 7), (8, 8), (9, 9), (10, 10), (11, 11), (12, 12), (13, 13), (14, 14), (15, 15), (16, 16), (17, 17), (18, 18), (19, 19), (20, 20), (21, 21), (22, 22), (23, 23)], default=0, verbose_name='Minimum zoom')),
                ('source', models.TextField(default='', verbose_name='Source')),
                ('screenshot', models.ImageField(help_text='Recommended size - Width=360 x Height=180', null=True, upload_to=djeym.utils.make_upload_path, validators=[djeym.utils.validate_image], verbose_name='Screenshot')),
                ('copyrights', models.TextField(default='', verbose_name='Copyrights')),
                ('site', models.URLField(blank=True, default='', verbose_name='Site')),
                ('apikey', models.CharField(blank=True, default='', help_text='API key or access_token', max_length=255, verbose_name='API Key')),
                ('note', models.TextField(blank=True, default='', verbose_name='Note')),
                ('slug', models.SlugField(blank=True, max_length=255, null=True, unique=True)),
            ],
            options={
                'verbose_name': 'Tile Source',
                'verbose_name_plural': 'Tile Sources',
                'ordering': ('title',),
            },
        ),
        migrations.CreateModel(
            name='SubCategoryPolyline',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='Subcategory - This is a characteristic feature that routes have.', max_length=60, verbose_name='Title')),
                ('category_icon', models.CharField(default='', help_text='http://materialdesignicons.com/ - Example: help OR mdi-help', max_length=255, verbose_name='Category Icon')),
                ('category_color', colorful.fields.RGBColorField(colors=('#F44336', '#C62828', '#E91E63', '#AD1457', '#9C27B0', '#6A1B9A', '#673AB7', '#4527A0', '#3F51B5', '#283593', '#2196F3', '#1565C0', '#0091EA', '#03A9F4', '#0277BD', '#00BCD4', '#00838F', '#009688', '#00695C', '#4CAF50', '#2E7D32', '#00C853', '#8BC34A', '#558B2F', '#CDDC39', '#9E9D24', '#FFEB3B', '#F9A825', '#FFC107', '#FF8F00', '#FF9800', '#EF6C00', '#FF5722', '#D84315', '#795548', '#4E342E', '#607D8B', '#37474F', '#FAFAFA', '#F5F5F5', '#EEEEEE', '#E0E0E0', '#BDBDBD', '#9E9E9E', '#757575', '#616161', '#424242', '#212121', '#000000', '#FFFFFF'), default='#0091EA', verbose_name='Category color')),
                ('position', models.PositiveIntegerField(db_index=True, default=0, editable=False)),
                ('active', models.BooleanField(default=True, verbose_name='Active category ?')),
                ('ymap', adminsortable.fields.SortableForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='subcategories_polyline', to='djeym.Map', verbose_name='Map')),
            ],
            options={
                'verbose_name': '_Subcategory of route',
                'verbose_name_plural': '_Subcategory of routes',
                'ordering': ('position',),
            },
        ),
        migrations.CreateModel(
            name='SubCategoryPolygon',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='Subcategory - This is a characteristic feature that the territories have.', max_length=60, verbose_name='Title')),
                ('category_icon', models.CharField(default='', help_text='http://materialdesignicons.com/ - Example: help OR mdi-help', max_length=255, verbose_name='Category Icon')),
                ('category_color', colorful.fields.RGBColorField(colors=('#F44336', '#C62828', '#E91E63', '#AD1457', '#9C27B0', '#6A1B9A', '#673AB7', '#4527A0', '#3F51B5', '#283593', '#2196F3', '#1565C0', '#0091EA', '#03A9F4', '#0277BD', '#00BCD4', '#00838F', '#009688', '#00695C', '#4CAF50', '#2E7D32', '#00C853', '#8BC34A', '#558B2F', '#CDDC39', '#9E9D24', '#FFEB3B', '#F9A825', '#FFC107', '#FF8F00', '#FF9800', '#EF6C00', '#FF5722', '#D84315', '#795548', '#4E342E', '#607D8B', '#37474F', '#FAFAFA', '#F5F5F5', '#EEEEEE', '#E0E0E0', '#BDBDBD', '#9E9E9E', '#757575', '#616161', '#424242', '#212121', '#000000', '#FFFFFF'), default='#0091EA', verbose_name='Category color')),
                ('position', models.PositiveIntegerField(db_index=True, default=0, editable=False)),
                ('active', models.BooleanField(default=True, verbose_name='Active category ?')),
                ('ymap', adminsortable.fields.SortableForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='subcategories_polygon', to='djeym.Map', verbose_name='Map')),
            ],
            options={
                'verbose_name': '_Subcategory of territory',
                'verbose_name_plural': '_Subcategory of territories',
                'ordering': ('position',),
            },
        ),
        migrations.CreateModel(
            name='SubCategoryPlacemark',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='Subcategory - This is a characteristic feature that markers have. Examples: Parking or Bicycle Parking or ATM or Cafe or Wi-Fi or Playground, etc.', max_length=60, verbose_name='Title')),
                ('category_icon', models.CharField(default='', help_text='http://materialdesignicons.com/ - Example: help OR mdi-help', max_length=255, verbose_name='Category Icon')),
                ('category_color', colorful.fields.RGBColorField(colors=('#F44336', '#C62828', '#E91E63', '#AD1457', '#9C27B0', '#6A1B9A', '#673AB7', '#4527A0', '#3F51B5', '#283593', '#2196F3', '#1565C0', '#0091EA', '#03A9F4', '#0277BD', '#00BCD4', '#00838F', '#009688', '#00695C', '#4CAF50', '#2E7D32', '#00C853', '#8BC34A', '#558B2F', '#CDDC39', '#9E9D24', '#FFEB3B', '#F9A825', '#FFC107', '#FF8F00', '#FF9800', '#EF6C00', '#FF5722', '#D84315', '#795548', '#4E342E', '#607D8B', '#37474F', '#FAFAFA', '#F5F5F5', '#EEEEEE', '#E0E0E0', '#BDBDBD', '#9E9E9E', '#757575', '#616161', '#424242', '#212121', '#000000', '#FFFFFF'), default='#0091EA', verbose_name='Category color')),
                ('position', models.PositiveIntegerField(db_index=True, default=0, editable=False)),
                ('active', models.BooleanField(default=True, verbose_name='Active category ?')),
                ('ymap', adminsortable.fields.SortableForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='subcategories_placemark', to='djeym.Map', verbose_name='Map')),
            ],
            options={
                'verbose_name': '_Subcategory of marker',
                'verbose_name_plural': '_Subcategory of markers',
                'ordering': ('position',),
            },
        ),
        migrations.CreateModel(
            name='Preset',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('position', models.PositiveSmallIntegerField(default=0, verbose_name='Position')),
                ('title', models.CharField(default='', max_length=60, verbose_name='Title')),
                ('icon', models.CharField(default='', help_text='http://materialdesignicons.com/ - Example: help OR mdi-help', max_length=255, verbose_name='Icon')),
                ('html', models.TextField(default='<p style="color:#E91E63;">iPreset</p>', verbose_name='Text | Html')),
                ('js', models.TextField(blank=True, default='', help_text='JavaScript and jQuery', verbose_name='JS')),
                ('description', models.TextField(default='<div style="color:#3F51B5;">Описание</div>\n<div style="color:#E91E63;">Description</div>', help_text='Brief description of the preset.', verbose_name='Description')),
                ('autoheader', models.BooleanField(default=False, verbose_name='Automatically add to end of header')),
                ('autobody', models.BooleanField(default=False, verbose_name='Automatically add to end of description')),
                ('autofooter', models.BooleanField(default=False, verbose_name='Automatically add to end of footer')),
                ('placemark', models.BooleanField(default=True, verbose_name='Markers')),
                ('polyline', models.BooleanField(default=True, verbose_name='Routes')),
                ('polygon', models.BooleanField(default=True, verbose_name='Territorys')),
                ('slug', models.SlugField(blank=True, max_length=255, null=True)),
                ('ymap', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='presets', to='djeym.Map', verbose_name='Map')),
            ],
            options={
                'verbose_name': 'Preset',
                'verbose_name_plural': 'Presets',
                'ordering': ('position',),
            },
        ),
        migrations.CreateModel(
            name='Polyline',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('header', ckeditor_uploader.fields.RichTextUploadingField(default='', verbose_name='Route name')),
                ('body', ckeditor_uploader.fields.RichTextUploadingField(blank=True, default='', verbose_name='Description of the geo object')),
                ('footer', ckeditor_uploader.fields.RichTextUploadingField(blank=True, default='', verbose_name='Footer')),
                ('stroke_width', models.PositiveIntegerField(default=5, verbose_name='Stroke width')),
                ('stroke_color', colorful.fields.RGBColorField(colors=('#F44336', '#C62828', '#E91E63', '#AD1457', '#9C27B0', '#6A1B9A', '#673AB7', '#4527A0', '#3F51B5', '#283593', '#2196F3', '#1565C0', '#0091EA', '#03A9F4', '#0277BD', '#00BCD4', '#00838F', '#009688', '#00695C', '#4CAF50', '#2E7D32', '#00C853', '#8BC34A', '#558B2F', '#CDDC39', '#9E9D24', '#FFEB3B', '#F9A825', '#FFC107', '#FF8F00', '#FF9800', '#EF6C00', '#FF5722', '#D84315', '#795548', '#4E342E', '#607D8B', '#37474F', '#FAFAFA', '#F5F5F5', '#EEEEEE', '#E0E0E0', '#BDBDBD', '#9E9E9E', '#757575', '#616161', '#424242', '#212121', '#000000', '#FFFFFF'), default='#00C853', verbose_name='Line color')),
                ('stroke_style', models.CharField(choices=[('solid', 'Solid line'), ('dash', 'Dash'), ('dashdot', 'Long dash-short dash'), ('dot', 'Dots'), ('longdash', 'Long dashes'), ('longdashdot', 'Extra long dash-dot'), ('longdashdotdot', 'Long dash-dot-dot'), ('shortdash', 'Short dashes'), ('shortdashdot', 'Dash-dot'), ('shortdashdotdot', 'Dash-dot-dot'), ('shortdot', 'Dots with double spacing')], default='solid', max_length=255, verbose_name='Line style')),
                ('stroke_opacity', models.CharField(choices=[('0', '0'), ('0.1', '0.1'), ('0.2', '0.2'), ('0.3', '0.3'), ('0.4', '0.4'), ('0.5', '0.5'), ('0.6', '0.6'), ('0.7', '0.7'), ('0.8', '0.8'), ('0.9', '0.9'), ('1', '1')], default='0.9', max_length=255, validators=[djeym.utils.validate_transparency], verbose_name='Opacity line')),
                ('coordinates', models.TextField(default='', verbose_name='Coordinates')),
                ('like', models.PositiveIntegerField(blank=True, default=0, verbose_name='Like')),
                ('dislike', models.PositiveIntegerField(blank=True, default=0, verbose_name='Dislike')),
                ('active', models.BooleanField(default=True, verbose_name='Active route ?')),
                ('json_code', models.TextField(blank=True, default='{"type": "Feature", "id": 0, "geometry": {"type": "LineString", "coordinates": []}, "properties": {"id": 0, "categoryID": 0, "subCategoryIDs": []}, "options": {"strokeWidth": 5, "strokeColor": "#00c853", "strokeStyle": "solid", "strokeOpacity": 0.9}}', editable=False, verbose_name='JSON')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('is_created', models.BooleanField(default=False, editable=False, verbose_name='Is created ?')),
                ('category', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='polylines', to='djeym.CategoryPlacemark', verbose_name='Category')),
                ('subcategories', models.ManyToManyField(blank=True, related_name='polylines', to='djeym.SubCategoryPlacemark', verbose_name='Subcategories')),
                ('ymap', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='polylines', to='djeym.Map', verbose_name='Map')),
            ],
            options={
                'verbose_name': 'Route',
                'verbose_name_plural': 'Routes',
                'ordering': ('-id',),
            },
        ),
        migrations.CreateModel(
            name='Polygon',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('header', ckeditor_uploader.fields.RichTextUploadingField(default='', verbose_name='Territory name')),
                ('body', ckeditor_uploader.fields.RichTextUploadingField(blank=True, default='', verbose_name='Description of the geo object')),
                ('footer', ckeditor_uploader.fields.RichTextUploadingField(blank=True, default='', verbose_name='Footer')),
                ('stroke_width', models.PositiveIntegerField(default=2, verbose_name='Stroke width')),
                ('stroke_color', colorful.fields.RGBColorField(colors=('#F44336', '#C62828', '#E91E63', '#AD1457', '#9C27B0', '#6A1B9A', '#673AB7', '#4527A0', '#3F51B5', '#283593', '#2196F3', '#1565C0', '#0091EA', '#03A9F4', '#0277BD', '#00BCD4', '#00838F', '#009688', '#00695C', '#4CAF50', '#2E7D32', '#00C853', '#8BC34A', '#558B2F', '#CDDC39', '#9E9D24', '#FFEB3B', '#F9A825', '#FFC107', '#FF8F00', '#FF9800', '#EF6C00', '#FF5722', '#D84315', '#795548', '#4E342E', '#607D8B', '#37474F', '#FAFAFA', '#F5F5F5', '#EEEEEE', '#E0E0E0', '#BDBDBD', '#9E9E9E', '#757575', '#616161', '#424242', '#212121', '#000000', '#FFFFFF'), default='#4caf50', verbose_name='Line color')),
                ('stroke_opacity', models.CharField(choices=[('0', '0'), ('0.1', '0.1'), ('0.2', '0.2'), ('0.3', '0.3'), ('0.4', '0.4'), ('0.5', '0.5'), ('0.6', '0.6'), ('0.7', '0.7'), ('0.8', '0.8'), ('0.9', '0.9'), ('1', '1')], default='0.9', max_length=255, validators=[djeym.utils.validate_transparency], verbose_name='Opacity line')),
                ('fill_color', colorful.fields.RGBColorField(colors=('#F44336', '#C62828', '#E91E63', '#AD1457', '#9C27B0', '#6A1B9A', '#673AB7', '#4527A0', '#3F51B5', '#283593', '#2196F3', '#1565C0', '#0091EA', '#03A9F4', '#0277BD', '#00BCD4', '#00838F', '#009688', '#00695C', '#4CAF50', '#2E7D32', '#00C853', '#8BC34A', '#558B2F', '#CDDC39', '#9E9D24', '#FFEB3B', '#F9A825', '#FFC107', '#FF8F00', '#FF9800', '#EF6C00', '#FF5722', '#D84315', '#795548', '#4E342E', '#607D8B', '#37474F', '#FAFAFA', '#F5F5F5', '#EEEEEE', '#E0E0E0', '#BDBDBD', '#9E9E9E', '#757575', '#616161', '#424242', '#212121', '#000000', '#FFFFFF'), default='#00C853', verbose_name='Fill color')),
                ('stroke_style', models.CharField(choices=[('solid', 'Solid line'), ('dash', 'Dash'), ('dashdot', 'Long dash-short dash'), ('dot', 'Dots'), ('longdash', 'Long dashes'), ('longdashdot', 'Extra long dash-dot'), ('longdashdotdot', 'Long dash-dot-dot'), ('shortdash', 'Short dashes'), ('shortdashdot', 'Dash-dot'), ('shortdashdotdot', 'Dash-dot-dot'), ('shortdot', 'Dots with double spacing')], default='solid', max_length=255, verbose_name='Line style')),
                ('fill_opacity', models.CharField(choices=[('0', '0'), ('0.1', '0.1'), ('0.2', '0.2'), ('0.3', '0.3'), ('0.4', '0.4'), ('0.5', '0.5'), ('0.6', '0.6'), ('0.7', '0.7'), ('0.8', '0.8'), ('0.9', '0.9'), ('1', '1')], default='0.9', max_length=255, validators=[djeym.utils.validate_transparency], verbose_name='Fill opacity')),
                ('coordinates', models.TextField(default='', verbose_name='Coordinates')),
                ('like', models.PositiveIntegerField(blank=True, default=0, verbose_name='Like')),
                ('dislike', models.PositiveIntegerField(blank=True, default=0, verbose_name='Dislike')),
                ('active', models.BooleanField(default=True, verbose_name='Active territory ?')),
                ('json_code', models.TextField(blank=True, default='{"type": "Feature", "id": 0, "geometry": {"type": "Polygon", "coordinates": []}, "properties": {"id": 0, "categoryID": 0, "subCategoryIDs": []}, "options": {"strokeWidth": 2, "strokeColor": "#00c853", "strokeStyle": "solid", "strokeOpacity": 0.9, "fillColor": "#00E676", "fillOpacity": 0.9}}', editable=False, verbose_name='JSON')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('is_created', models.BooleanField(default=False, editable=False, verbose_name='Is created ?')),
                ('category', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='polygons', to='djeym.CategoryPlacemark', verbose_name='Category')),
                ('subcategories', models.ManyToManyField(blank=True, related_name='polygons', to='djeym.SubCategoryPlacemark', verbose_name='Subcategories')),
                ('ymap', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='polygons', to='djeym.Map', verbose_name='Map')),
            ],
            options={
                'verbose_name': 'Territorys',
                'verbose_name_plural': 'Territorys',
                'ordering': ('-id',),
            },
        ),
        migrations.CreateModel(
            name='Placemark',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('header', ckeditor_uploader.fields.RichTextUploadingField(default='', verbose_name='Place name')),
                ('body', ckeditor_uploader.fields.RichTextUploadingField(blank=True, default='', verbose_name='Description of the geo object')),
                ('user_image', models.ImageField(blank=True, editable=False, help_text='Image from the user to description the geo-object (width = 966px, quality = 40).', null=True, upload_to=djeym.utils.make_upload_path, verbose_name='Image from user')),
                ('footer', ckeditor_uploader.fields.RichTextUploadingField(blank=True, default='', verbose_name='Footer')),
                ('icon_slug', models.SlugField(max_length=255, null=True, verbose_name='Icon (slug)')),
                ('coordinates', models.CharField(default='[0,0]', max_length=255, verbose_name='Coordinates')),
                ('like', models.PositiveIntegerField(blank=True, default=0, verbose_name='Like')),
                ('dislike', models.PositiveIntegerField(blank=True, default=0, verbose_name='Dislike')),
                ('active', models.BooleanField(default=True, verbose_name='Active placemark ?')),
                ('user_email', models.EmailField(blank=True, help_text="To automatically send a one-time message if the user's marker has been moderated and activated.", max_length=254, null=True, verbose_name='User Email')),
                ('user_ip', models.GenericIPAddressField(blank=True, help_text='Ban user if vandalism is to be prevented.', null=True, verbose_name='User IP')),
                ('is_user_marker', models.BooleanField(default=False, verbose_name='Is user marker ?')),
                ('json_code', models.TextField(blank=True, default='{"type": "Feature", "id": 0, "geometry": {"type": "Point", "coordinates": []}, "properties": {"id": 0, "categoryID": 0, "subCategoryIDs": [], "iconSlug": ""}, "options": {"iconLayout": "default#image", "iconImageHref": "", "iconImageSize": [], "iconImageOffset": []}}', editable=False, verbose_name='JSON')),
                ('created_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('is_created', models.BooleanField(default=False, editable=False, verbose_name='Is created ?')),
                ('is_sended_admin_email', models.BooleanField(default=False, editable=False, verbose_name='Is sent to admin by email ?')),
                ('is_sended_user_email', models.BooleanField(default=False, editable=False, verbose_name='Is sended user email ?')),
                ('category', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='placemarks', to='djeym.CategoryPlacemark', verbose_name='Category')),
                ('subcategories', models.ManyToManyField(blank=True, related_name='placemarks', to='djeym.SubCategoryPlacemark', verbose_name='Subcategories')),
                ('ymap', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='placemarks', to='djeym.Map', verbose_name='Map')),
            ],
            options={
                'verbose_name': 'Marker',
                'verbose_name_plural': 'Markers',
                'ordering': ('-id',),
            },
        ),
        migrations.CreateModel(
            name='MarkerIcon',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('svg', models.FileField(help_text='Only SVG files.', null=True, upload_to=djeym.utils.make_upload_path, validators=[djeym.utils.validate_svg], verbose_name='Icon')),
                ('title', models.CharField(default='', help_text='Example: Airport', max_length=60, verbose_name='Icon name')),
                ('size_width', models.PositiveSmallIntegerField(default=0, verbose_name='Icon width')),
                ('size_height', models.PositiveSmallIntegerField(default=0, verbose_name='Icon height')),
                ('offset_x', models.DecimalField(decimal_places=1, default=Decimal('0.0'), help_text='Left, right - First time is automatically calculated.', max_digits=3, verbose_name='Offset by axis - X')),
                ('offset_y', models.DecimalField(decimal_places=1, default=Decimal('0.0'), help_text='Up, down - First time is automatically calculated.', max_digits=3, verbose_name='Offset by axis - Y')),
                ('active', models.BooleanField(default=True, help_text='If the project uses 2-3 icons, it makes sense to disable the rest to optimize the download.', verbose_name='Active icon ?')),
                ('slug', models.SlugField(blank=True, max_length=255, null=True)),
                ('icon_collection', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='icons', to='djeym.IconCollection', verbose_name='Icon Collection')),
            ],
            options={
                'verbose_name': 'Icons for markers',
                'verbose_name_plural': 'Icons for markers',
                'ordering': ('title', 'id'),
            },
        ),
        migrations.CreateModel(
            name='MapControls',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('geolocation', models.BooleanField(default=True, verbose_name='Geolocation')),
                ('search', models.BooleanField(default=True, verbose_name='Search by map')),
                ('provider', models.BooleanField(default=True, verbose_name='Search by organization name')),
                ('route', models.BooleanField(default=True, verbose_name='Routing panel')),
                ('traffic', models.BooleanField(default=True, verbose_name='Traffic jams')),
                ('typeselector', models.BooleanField(default=True, verbose_name='Map Layer Switch')),
                ('fullscreen', models.BooleanField(default=True, verbose_name='Full screen mode')),
                ('zoom', models.BooleanField(default=True, verbose_name='Zoom')),
                ('ruler', models.BooleanField(default=True, verbose_name='Ruler')),
                ('ymap', models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='map_controls', to='djeym.Map', verbose_name='Map')),
            ],
            options={
                'verbose_name_plural': 'Map Controls',
            },
        ),
        migrations.AddField(
            model_name='map',
            name='tile',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='ymap', to='djeym.TileSource', verbose_name='Tile Source'),
        ),
        migrations.CreateModel(
            name='JsonSettings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('editor', models.TextField(default='{}', verbose_name='All settings for editor page')),
                ('front', models.TextField(default='{}', verbose_name='All settings for front page')),
                ('ymap', models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='json_settings', to='djeym.Map', verbose_name='Map')),
            ],
        ),
        migrations.CreateModel(
            name='HeatPoint',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, default='', max_length=60, verbose_name='Place name')),
                ('weight', models.PositiveIntegerField(blank=True, default=0, help_text='Examples: Average property value on a district site or Number of cameras installed on a building, etc.', verbose_name='Weight')),
                ('coordinates', models.CharField(default='[0,0]', max_length=255, verbose_name='Coordinates')),
                ('active', models.BooleanField(default=True, verbose_name='Active heat point ?')),
                ('slug', models.SlugField(blank=True, max_length=255, null=True, unique=True)),
                ('json_code', models.TextField(blank=True, default='{"type": "Feature", "id": 0, "geometry": {"type": "Point", "coordinates": []}, "properties": {"weight": 0}}', editable=False, verbose_name='JSON')),
                ('is_created', models.BooleanField(default=False, editable=False, verbose_name='Is created ?')),
                ('ymap', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='heat_points', to='djeym.Map', verbose_name='Map')),
            ],
            options={
                'verbose_name': 'Heat points',
                'verbose_name_plural': 'Heat points',
            },
        ),
        migrations.CreateModel(
            name='HeatmapSettings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('radius', models.PositiveIntegerField(default=10, verbose_name='Point radius of influence (px)')),
                ('dissipating', models.BooleanField(default=False, help_text="Enable - Disperse points on higher zoom levels according to radius. Disable - Don't disperse.", verbose_name='Disperse points')),
                ('opacity', models.CharField(choices=[('0', '0'), ('0.1', '0.1'), ('0.2', '0.2'), ('0.3', '0.3'), ('0.4', '0.4'), ('0.5', '0.5'), ('0.6', '0.6'), ('0.7', '0.7'), ('0.8', '0.8'), ('0.9', '0.9'), ('1', '1')], default='0.8', max_length=255, validators=[djeym.utils.validate_transparency], verbose_name='Transparency of heat layer')),
                ('intensity', models.CharField(choices=[('0', '0'), ('0.1', '0.1'), ('0.2', '0.2'), ('0.3', '0.3'), ('0.4', '0.4'), ('0.5', '0.5'), ('0.6', '0.6'), ('0.7', '0.7'), ('0.8', '0.8'), ('0.9', '0.9'), ('1', '1')], default='0.2', max_length=255, validators=[djeym.utils.validate_transparency], verbose_name='Intensity of median point')),
                ('gradient_color1', colorful.fields.RGBColorField(colors=('#F44336', '#C62828', '#E91E63', '#AD1457', '#9C27B0', '#6A1B9A', '#673AB7', '#4527A0', '#3F51B5', '#283593', '#2196F3', '#1565C0', '#0091EA', '#03A9F4', '#0277BD', '#00BCD4', '#00838F', '#009688', '#00695C', '#4CAF50', '#2E7D32', '#00C853', '#8BC34A', '#558B2F', '#CDDC39', '#9E9D24', '#FFEB3B', '#F9A825', '#FFC107', '#FF8F00', '#FF9800', '#EF6C00', '#FF5722', '#D84315', '#795548', '#4E342E', '#607D8B', '#37474F', '#FAFAFA', '#F5F5F5', '#EEEEEE', '#E0E0E0', '#BDBDBD', '#9E9E9E', '#757575', '#616161', '#424242', '#212121', '#000000', '#FFFFFF'), default='#66BB6A', verbose_name='gradient color 1')),
                ('gradient_color2', colorful.fields.RGBColorField(colors=('#F44336', '#C62828', '#E91E63', '#AD1457', '#9C27B0', '#6A1B9A', '#673AB7', '#4527A0', '#3F51B5', '#283593', '#2196F3', '#1565C0', '#0091EA', '#03A9F4', '#0277BD', '#00BCD4', '#00838F', '#009688', '#00695C', '#4CAF50', '#2E7D32', '#00C853', '#8BC34A', '#558B2F', '#CDDC39', '#9E9D24', '#FFEB3B', '#F9A825', '#FFC107', '#FF8F00', '#FF9800', '#EF6C00', '#FF5722', '#D84315', '#795548', '#4E342E', '#607D8B', '#37474F', '#FAFAFA', '#F5F5F5', '#EEEEEE', '#E0E0E0', '#BDBDBD', '#9E9E9E', '#757575', '#616161', '#424242', '#212121', '#000000', '#FFFFFF'), default='#FDD835', verbose_name='gradient color 2')),
                ('gradient_color3', colorful.fields.RGBColorField(colors=('#F44336', '#C62828', '#E91E63', '#AD1457', '#9C27B0', '#6A1B9A', '#673AB7', '#4527A0', '#3F51B5', '#283593', '#2196F3', '#1565C0', '#0091EA', '#03A9F4', '#0277BD', '#00BCD4', '#00838F', '#009688', '#00695C', '#4CAF50', '#2E7D32', '#00C853', '#8BC34A', '#558B2F', '#CDDC39', '#9E9D24', '#FFEB3B', '#F9A825', '#FFC107', '#FF8F00', '#FF9800', '#EF6C00', '#FF5722', '#D84315', '#795548', '#4E342E', '#607D8B', '#37474F', '#FAFAFA', '#F5F5F5', '#EEEEEE', '#E0E0E0', '#BDBDBD', '#9E9E9E', '#757575', '#616161', '#424242', '#212121', '#000000', '#FFFFFF'), default='#EF5350', verbose_name='gradient color 3')),
                ('gradient_color4', colorful.fields.RGBColorField(colors=('#F44336', '#C62828', '#E91E63', '#AD1457', '#9C27B0', '#6A1B9A', '#673AB7', '#4527A0', '#3F51B5', '#283593', '#2196F3', '#1565C0', '#0091EA', '#03A9F4', '#0277BD', '#00BCD4', '#00838F', '#009688', '#00695C', '#4CAF50', '#2E7D32', '#00C853', '#8BC34A', '#558B2F', '#CDDC39', '#9E9D24', '#FFEB3B', '#F9A825', '#FFC107', '#FF8F00', '#FF9800', '#EF6C00', '#FF5722', '#D84315', '#795548', '#4E342E', '#607D8B', '#37474F', '#FAFAFA', '#F5F5F5', '#EEEEEE', '#E0E0E0', '#BDBDBD', '#9E9E9E', '#757575', '#616161', '#424242', '#212121', '#000000', '#FFFFFF'), default='#B71C1C', verbose_name='gradient color 4')),
                ('active', models.BooleanField(default=False, verbose_name='Active Heatmap ?')),
                ('ymap', models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='heatmap_settings', to='djeym.Map', verbose_name='Map')),
            ],
            options={
                'verbose_name': 'Heatmap',
                'verbose_name_plural': 'Heatmap',
            },
        ),
        migrations.CreateModel(
            name='GeneralSettings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('clustering_edit', models.BooleanField(default=True, verbose_name='Сlustering in editor')),
                ('clustering_site', models.BooleanField(default=True, verbose_name='Сlustering on site')),
                ('cluster_layout', models.CharField(choices=[('cluster#balloonTwoColumns', 'Two Columns'), ('cluster#balloonCarousel', 'Carousel')], default='cluster#balloonTwoColumns', max_length=255, verbose_name='Layout for the cluster information window')),
                ('cluster_icon_content', models.BooleanField(default=True, verbose_name='Display the number of objects in the cluster icon')),
                ('cluster_icon_content_bg_color', colorful.fields.RGBColorField(colors=('#F44336', '#C62828', '#E91E63', '#AD1457', '#9C27B0', '#6A1B9A', '#673AB7', '#4527A0', '#3F51B5', '#283593', '#2196F3', '#1565C0', '#0091EA', '#03A9F4', '#0277BD', '#00BCD4', '#00838F', '#009688', '#00695C', '#4CAF50', '#2E7D32', '#00C853', '#8BC34A', '#558B2F', '#CDDC39', '#9E9D24', '#FFEB3B', '#F9A825', '#FFC107', '#FF8F00', '#FF9800', '#EF6C00', '#FF5722', '#D84315', '#795548', '#4E342E', '#607D8B', '#37474F', '#FAFAFA', '#F5F5F5', '#EEEEEE', '#E0E0E0', '#BDBDBD', '#9E9E9E', '#757575', '#616161', '#424242', '#212121', '#000000', '#FFFFFF'), default='#F5F5F5', verbose_name='Background color')),
                ('cluster_icon_content_txt_color', colorful.fields.RGBColorField(colors=('#F44336', '#C62828', '#E91E63', '#AD1457', '#9C27B0', '#6A1B9A', '#673AB7', '#4527A0', '#3F51B5', '#283593', '#2196F3', '#1565C0', '#0091EA', '#03A9F4', '#0277BD', '#00BCD4', '#00838F', '#009688', '#00695C', '#4CAF50', '#2E7D32', '#00C853', '#8BC34A', '#558B2F', '#CDDC39', '#9E9D24', '#FFEB3B', '#F9A825', '#FFC107', '#FF8F00', '#FF9800', '#EF6C00', '#FF5722', '#D84315', '#795548', '#4E342E', '#607D8B', '#37474F', '#FAFAFA', '#F5F5F5', '#EEEEEE', '#E0E0E0', '#BDBDBD', '#9E9E9E', '#757575', '#616161', '#424242', '#212121', '#000000', '#FFFFFF'), default='#212121', verbose_name='Text color')),
                ('controls_color', colorful.fields.RGBColorField(colors=('#F44336', '#C62828', '#E91E63', '#AD1457', '#9C27B0', '#6A1B9A', '#673AB7', '#4527A0', '#3F51B5', '#283593', '#2196F3', '#1565C0', '#0091EA', '#03A9F4', '#0277BD', '#00BCD4', '#00838F', '#009688', '#00695C', '#4CAF50', '#2E7D32', '#00C853', '#8BC34A', '#558B2F', '#CDDC39', '#9E9D24', '#FFEB3B', '#F9A825', '#FFC107', '#FF8F00', '#FF9800', '#EF6C00', '#FF5722', '#D84315', '#795548', '#4E342E', '#607D8B', '#37474F', '#FAFAFA', '#F5F5F5', '#EEEEEE', '#E0E0E0', '#BDBDBD', '#9E9E9E', '#757575', '#616161', '#424242', '#212121', '#000000', '#FFFFFF'), default='#FFA000', verbose_name='Color of controls')),
                ('buttons_text_color', colorful.fields.RGBColorField(colors=('#F44336', '#C62828', '#E91E63', '#AD1457', '#9C27B0', '#6A1B9A', '#673AB7', '#4527A0', '#3F51B5', '#283593', '#2196F3', '#1565C0', '#0091EA', '#03A9F4', '#0277BD', '#00BCD4', '#00838F', '#009688', '#00695C', '#4CAF50', '#2E7D32', '#00C853', '#8BC34A', '#558B2F', '#CDDC39', '#9E9D24', '#FFEB3B', '#F9A825', '#FFC107', '#FF8F00', '#FF9800', '#EF6C00', '#FF5722', '#D84315', '#795548', '#4E342E', '#607D8B', '#37474F', '#FAFAFA', '#F5F5F5', '#EEEEEE', '#E0E0E0', '#BDBDBD', '#9E9E9E', '#757575', '#616161', '#424242', '#212121', '#000000', '#FFFFFF'), default='#FAFAFA', verbose_name='Text color on buttons')),
                ('theme_type', models.CharField(choices=[('light', 'Light'), ('dark', 'Dark')], default='light', max_length=255, verbose_name='Theme type - light | dark')),
                ('roundtheme', models.BooleanField(default=False, verbose_name='Round theme of controls')),
                ('panorama', models.BooleanField(default=True, verbose_name='Panorama - on|off')),
                ('width_panel_editor', models.PositiveSmallIntegerField(default=380, verbose_name='Width for panel of editor')),
                ('width_panel_front', models.PositiveSmallIntegerField(default=380, verbose_name='Width for panel of front')),
                ('open_panel_front', models.BooleanField(default=False, verbose_name='Open panel automatically')),
                ('img_bg_panel_front', models.ImageField(blank=True, null=True, upload_to=djeym.utils.make_upload_path, verbose_name='Background image for the site panel')),
                ('tinting_panel_front', models.CharField(default='#00000000', max_length=9, verbose_name='Background under controls of panel')),
                ('hide_group_name_panel_front', models.BooleanField(default=False, verbose_name='Hide group names (Categories, Subcategories)')),
                ('width_map_front', models.CharField(default='100%', max_length=255, verbose_name='Width of Map for Front page')),
                ('height_map_front', models.CharField(default='600px', max_length=255, verbose_name='Height of Map for Front page')),
                ('ymap', models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='general_settings', to='djeym.Map', verbose_name='Map')),
            ],
            options={
                'verbose_name': 'General settings',
                'verbose_name_plural': 'General settings',
            },
        ),
        migrations.CreateModel(
            name='CategoryPolyline',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='Examples: Tourist routes or Marathon routes etc.', max_length=60, verbose_name='Title')),
                ('category_icon', models.CharField(default='', help_text='http://materialdesignicons.com/ - Example: help OR mdi-help', max_length=255, verbose_name='Category Icon')),
                ('category_color', colorful.fields.RGBColorField(colors=('#F44336', '#C62828', '#E91E63', '#AD1457', '#9C27B0', '#6A1B9A', '#673AB7', '#4527A0', '#3F51B5', '#283593', '#2196F3', '#1565C0', '#0091EA', '#03A9F4', '#0277BD', '#00BCD4', '#00838F', '#009688', '#00695C', '#4CAF50', '#2E7D32', '#00C853', '#8BC34A', '#558B2F', '#CDDC39', '#9E9D24', '#FFEB3B', '#F9A825', '#FFC107', '#FF8F00', '#FF9800', '#EF6C00', '#FF5722', '#D84315', '#795548', '#4E342E', '#607D8B', '#37474F', '#FAFAFA', '#F5F5F5', '#EEEEEE', '#E0E0E0', '#BDBDBD', '#9E9E9E', '#757575', '#616161', '#424242', '#212121', '#000000', '#FFFFFF'), default='#00C853', verbose_name='Category color')),
                ('position', models.PositiveIntegerField(db_index=True, default=0, editable=False)),
                ('active', models.BooleanField(default=True, verbose_name='Active category ?')),
                ('ymap', adminsortable.fields.SortableForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='categories_polyline', to='djeym.Map', verbose_name='Map')),
            ],
            options={
                'verbose_name': '-Category of route',
                'verbose_name_plural': '-Category of routes',
                'ordering': ('position',),
            },
        ),
        migrations.CreateModel(
            name='CategoryPolygon',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='The name of the aggregate of any geographic area. Examples: Honduran cities or Hawaiian beaches or Residential complexes in Antarctica.', max_length=60, verbose_name='Title')),
                ('category_icon', models.CharField(default='', help_text='http://materialdesignicons.com/ - Example: help OR mdi-help', max_length=255, verbose_name='Category Icon')),
                ('category_color', colorful.fields.RGBColorField(colors=('#F44336', '#C62828', '#E91E63', '#AD1457', '#9C27B0', '#6A1B9A', '#673AB7', '#4527A0', '#3F51B5', '#283593', '#2196F3', '#1565C0', '#0091EA', '#03A9F4', '#0277BD', '#00BCD4', '#00838F', '#009688', '#00695C', '#4CAF50', '#2E7D32', '#00C853', '#8BC34A', '#558B2F', '#CDDC39', '#9E9D24', '#FFEB3B', '#F9A825', '#FFC107', '#FF8F00', '#FF9800', '#EF6C00', '#FF5722', '#D84315', '#795548', '#4E342E', '#607D8B', '#37474F', '#FAFAFA', '#F5F5F5', '#EEEEEE', '#E0E0E0', '#BDBDBD', '#9E9E9E', '#757575', '#616161', '#424242', '#212121', '#000000', '#FFFFFF'), default='#00C853', verbose_name='Category color')),
                ('position', models.PositiveIntegerField(db_index=True, default=0, editable=False)),
                ('active', models.BooleanField(default=True, verbose_name='Active category ?')),
                ('ymap', adminsortable.fields.SortableForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='categories_polygon', to='djeym.Map', verbose_name='Map')),
            ],
            options={
                'verbose_name': '-Category of territory',
                'verbose_name_plural': '-Category of territories',
                'ordering': ('position',),
            },
        ),
        migrations.AddField(
            model_name='categoryplacemark',
            name='ymap',
            field=adminsortable.fields.SortableForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='categories_placemark', to='djeym.Map', verbose_name='Map'),
        ),
    ]



# -*- coding: utf-8 -*-
from adminsortable.admin import NonSortableParentAdmin, SortableTabularInline
from django.contrib import admin
from django.db import models

from .forms import CenterMapForm, OffsetMarkerIconForm
from .models import (BlockedIP, CategoryPlacemark, CategoryPolygon,
                     CategoryPolyline, ClusterIcon, HeatPoint, IconCollection,
                     LoadIndicator, Map, MarkerIcon, Placemark, Polygon,
                     Polyline, Preset, Statistics, SubCategoryPlacemark,
                     SubCategoryPolygon, SubCategoryPolyline, TileSource, Status)
from .widgets import AdminFileThumbWidget


@admin.register(TileSource)
class TileSourceAdmin(admin.ModelAdmin):
    # ckeditor_change_form.html - Used by default.
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    change_list_template = 'djeym/admin/tile_source_change_list.html'
    list_display = ('title', 'admin_thumbnail', 'maxzoom', 'minzoom', 'slug')
    list_display_links = ('title', 'admin_thumbnail')
    readonly_fields = ('slug',)

    formfield_overrides = {
        models.ImageField: {'widget': AdminFileThumbWidget()},
    }

    class Media:
        css = {
            'all': [
                '/static/djeym/css/djeym_admin.min.css',
            ]
        }

        js = [
            '/static/djeym/js/jquery.min.js',
            '/static/djeym/js/import_export.min.js'
        ]


@admin.register(Statistics)
class StatisticsAdmin(admin.ModelAdmin):
    # ckeditor_change_form.html - Used by default.
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    list_display = ('obj_type', 'obj_id', 'ip', 'timestamp')
    readonly_fields = ('likes',)

    class Media:
        css = {
            'all': ['/static/djeym/css/djeym_admin.min.css']
        }


@admin.register(BlockedIP)
class BlockedIPAdmin(admin.ModelAdmin):
    # ckeditor_change_form.html - Used by default.
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    list_display = ('ip', 'timestamp')

    class Media:
        css = {
            'all': ['/static/djeym/css/djeym_admin.min.css']
        }


class PresetInline(admin.StackedInline):
    model = Preset
    extra = 0
    exclude = ('slug',)
    classes = ['collapse']


class CategoryPlacemarkInline(SortableTabularInline):
    model = CategoryPlacemark
    extra = 0
    classes = ['collapse']


class SubCategoryPlacemarkInline(SortableTabularInline):
    model = SubCategoryPlacemark
    extra = 0
    classes = ['collapse']


class CategoryPolylineInline(SortableTabularInline):
    model = CategoryPolyline
    extra = 0
    classes = ['collapse']


class SubCategoryPolylineInline(SortableTabularInline):
    model = SubCategoryPolyline
    extra = 0
    classes = ['collapse']


class CategoryPolygonInline(SortableTabularInline):
    model = CategoryPolygon
    extra = 0
    classes = ['collapse']


class SubCategoryPolygonInline(SortableTabularInline):
    model = SubCategoryPolygon
    extra = 0
    classes = ['collapse']


@admin.register(Map)
class MapAdmin(NonSortableParentAdmin):
    form = CenterMapForm
    change_form_template_extends = 'djeym/admin/center_map_change_form.html'
    list_display = ('title', 'slug', 'get_cluster', 'get_icon_collection',
                    'get_tile_screenshot', 'get_load_indicator', 'get_status_heatmap',
                    'zoom', 'active')
    list_display_links = ('title', 'get_cluster', 'get_status_heatmap',
                          'get_icon_collection', 'get_tile_screenshot', 'get_load_indicator')
    list_editable = ('active',)
    inlines = (PresetInline, CategoryPlacemarkInline, SubCategoryPlacemarkInline,
               CategoryPolylineInline, SubCategoryPolylineInline, CategoryPolygonInline,
               SubCategoryPolygonInline)

    class Media:
        css = {
            'all': ['/static/djeym/css/djeym_admin.min.css']
        }

        js = [
            '/static/djeym/js/jquery.min.js',
            '/static/djeym/js/admin_view_icons.min.js'
        ]


@admin.register(Placemark)
class PlacemarkAdmin(admin.ModelAdmin):
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    list_display = ('__str__', 'ymap', 'category', 'active')
    list_filter = ('ymap',)
    list_editable = ('active',)
    filter_horizontal = ('subcategories',)

    formfield_overrides = {
        models.ImageField: {'widget': AdminFileThumbWidget()},
    }

    class Media:
        css = {
            'all': [
                '/static/djeym/css/djeym_admin.min.css',
            ]
        }

        js = (
            '/static/djeym/js/jquery.min.js',
            '/static/djeym/js/ckeditor_resize_image.min.js',
            '/static/djeym/js/admin_icon_collection.min.js',
            '/static/djeym/js/admin_block_ip.min.js'
        )


@admin.register(Polyline)
class PolylineAdmin(admin.ModelAdmin):
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    list_display = ('__str__', 'active')
    list_filter = ('ymap',)
    list_editable = ('active',)
    filter_horizontal = ('subcategories',)

    class Media:
        css = {
            'all': [
                '/static/djeym/css/djeym_admin.min.css',
            ]
        }

        js = (
            '/static/djeym/js/jquery.min.js',
            '/static/djeym/js/ckeditor_resize_image.min.js',
        )


@admin.register(Polygon)
class PolygonAdmin(admin.ModelAdmin):
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    list_display = ('__str__', 'active')
    list_filter = ('ymap',)
    list_editable = ('active',)
    filter_horizontal = ('subcategories',)

    class Media:
        css = {
            'all': [
                '/static/djeym/css/djeym_admin.min.css',
            ]
        }

        js = (
            '/static/djeym/js/jquery.min.js',
            '/static/djeym/js/ckeditor_resize_image.min.js',
        )


@admin.register(HeatPoint)
class HeatPointAdmin(admin.ModelAdmin):
    # ckeditor_change_form.html - Used by default.
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    list_display = ('title', 'weight', 'slug', 'active')
    list_editable = ('active',)
    list_filter = ('ymap',)
    readonly_fields = ('slug',)
    search_fields = ('title',)

    class Media:
        css = {
            'all': [
                '/static/djeym/css/djeym_admin.min.css',
            ]
        }


@admin.register(ClusterIcon)
class ClusterIconAdmin(admin.ModelAdmin):
    # ckeditor_change_form.html - Used by default.
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    list_display = ('title', 'admin_thumbnail')
    list_display_links = ('title', 'admin_thumbnail')
    readonly_fields = ('size_width', 'size_height', 'offset_x', 'offset_y')

    formfield_overrides = {
        models.FileField: {'widget': AdminFileThumbWidget()},
    }

    class Media:
        css = {
            'all': [
                '/static/djeym/css/djeym_admin.min.css',
            ]
        }

        js = [
            '/static/djeym/js/jquery.min.js',
            '/static/djeym/js/get_icon_name.min.js'
        ]


@admin.register(IconCollection)
class IconCollectionAdmin(admin.ModelAdmin):
    # ckeditor_change_form.html - Used by default.
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    change_list_template = 'djeym/admin/icon_collection_change_list.html'
    list_display = ('title', 'admin_thumbnail', 'get_export_file_btn',
                    'get_icon_count', 'get_count_of_active_icons')
    list_display_links = ('title', 'admin_thumbnail')
    readonly_fields = ('slug',)

    class Media:
        css = {
            'all': [
                '/static/djeym/css/djeym_admin.min.css',
            ]
        }

        js = [
            '/static/djeym/js/jquery.min.js',
            '/static/djeym/js/import_export.min.js'
        ]


@admin.register(MarkerIcon)
class MarkerIconAdmin(admin.ModelAdmin):
    form = OffsetMarkerIconForm
    change_form_template = 'djeym/admin/check_icon_offset_change_form.html'
    list_display = ('title', 'admin_thumbnail', 'get_collection_name',
                    'active', 'slug')
    list_display_links = ('title', 'admin_thumbnail')
    readonly_fields = ('size_width', 'size_height', 'slug')
    list_editable = ('active',)
    list_filter = ('icon_collection',)

    formfield_overrides = {
        models.FileField: {'widget': AdminFileThumbWidget()},
    }

    class Media:
        css = {
            'all': [
                '/static/djeym/css/djeym_admin.min.css',
            ]
        }

        js = [
            '/static/djeym/js/jquery.min.js',
            '/static/djeym/plugins/jquery_mousewheel/jquery.mousewheel.min.js',
            '/static/djeym/js/get_icon_name.min.js'
        ]


@admin.register(LoadIndicator)
class LoadIndicatorAdmin(admin.ModelAdmin):
    # ckeditor_change_form.html - Used by default.
    change_form_template = 'djeym/admin/ckeditor_change_form.html'

    list_display = ('title', 'admin_thumbnail', 'slug')
    list_display_links = ('title', 'admin_thumbnail')
    readonly_fields = ('slug',)

    formfield_overrides = {
        models.FileField: {'widget': AdminFileThumbWidget()},
    }

    class Media:
        css = {
            'all': [
                '/static/djeym/css/djeym_admin.min.css',
            ]
        }

        js = [
            '/static/djeym/js/jquery.min.js',
            '/static/djeym/js/get_icon_name.min.js'
        ]


@admin.register(Status)
class StatusAdmin(admin.ModelAdmin):
    # ckeditor_change_form.html - Used by default.
    change_form_template = 'djeym/admin/ckeditor_change_form.html'
    list_display = ('title', 'slug')

    class Media:
        css = {
            'all': ['/static/djeym/css/djeym_admin.min.css']
        }



# -*- coding: utf-8 -*-
from django.conf import settings
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.shortcuts import redirect
from django.utils.decorators import method_decorator
from django.utils.translation import gettext_lazy as _


class StaffRequiredMixin:
    """
    Mixin which requires that the authenticated user is a staff member
    (i.e. `is_staff` is True).
    """
    # login_required(redirect_field_name='next', login_url=None)
    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_staff:
            messages.error(
                request,
                _('You do not have the permission required to perform the requested operation.'))
            return redirect(settings.LOGIN_URL)
        return super(StaffRequiredMixin, self).dispatch(request, *args, **kwargs)



# -*- coding: utf-8 -*-
from decimal import Decimal

from django.test import TestCase

from .models import (CategoryPlacemark, CategoryPolygon, CategoryPolyline,
                     ClusterIcon, GeneralSettings, HeatmapSettings, HeatPoint,
                     IconCollection, Map, MapControls, MarkerIcon, Placemark,
                     Polygon, Polyline, Preset, Statistics,
                     SubCategoryPlacemark, TileSource)

'''
class ModelsTest(TestCase):
    """
    Testing models - Integrity check on the default state.
    Тестирование моделей - Проверка целостности по дефолтному состоянию.
    """

    def setUp(self):
        # Map
        self.title_map = "Супермаркеты Харькова"
        self.slug_map = "supermarkety-kharkova"

        # CategoryPlacemark
        self.title_category_placemark_1 = "Шевченковский район"
        self.title_category_placemark_2 = "Холодногорский район"

        # SubCategoryPlacemark
        self.title_subcategory_placemark_1 = "Автопарковка"
        self.title_subcategory_placemark_2 = "Банкомат"

        # CategoryPolyline
        self.title_category_polyline_1 = "Маршруты марафонов"
        self.title_category_polyline_2 = "Туристические маршруты"

        # CategoryPolygon
        self.title_category_polygon_1 = "Ботанический Сад"
        self.title_category_polygon_2 = "Новобаварский район"

        # Placemark
        self.header_placemark_1 = "<p>Test placemark 1</p>"
        self.header_placemark_2 = "<p>Test placemark 2</p>"

        # Polyline
        self.header_polyline_1 = "<p>Test polyline 1</p>"
        self.header_polyline_2 = "<p>Test polyline 2</p>"

        # Polygon
        self.header_polygon_1 = "<p>Test polygon 1</p>"
        self.header_polygon_2 = "<p>Test polygon 2</p>"

        # ClusterIcon
        self.title_cluster_icon = "Test Cluster Icon"

        # IconCollection
        self.title_icon_collection = "Test Icon Collection"

        # MarkerIcon
        self.custom_marker_icon_1 = "Test Custom Marker Icon 1"
        self.custom_marker_icon_2 = "Test Custom Marker Icon 2"

        # TileSource
        self.title_tile = "OpenTopoMap"

    def test_tile_source(self):
        """
        Test the model of the TileSource with the default settings.
        Тестирование модели TileSource с настройками по умолчанию.
        """
        title_tile = self.title_tile = "OpenTopoMap"
        test_tile = TileSource.objects.create(title=title_tile)

        self.assertTrue(TileSource._meta.get_field('title').unique)
        self.assertFalse(TileSource._meta.get_field('title').blank)
        self.assertEqual(TileSource._meta.get_field('title').default, "")

        self.assertFalse(TileSource._meta.get_field('maxzoom').blank)
        self.assertEqual(TileSource._meta.get_field('maxzoom').default, 12)

        self.assertFalse(TileSource._meta.get_field('minzoom').blank)
        self.assertEqual(TileSource._meta.get_field('minzoom').default, 0)

        self.assertFalse(TileSource._meta.get_field('source').blank)
        self.assertEqual(TileSource._meta.get_field('source').default, "")

        self.assertFalse(TileSource._meta.get_field('screenshot').blank)
        self.assertTrue(TileSource._meta.get_field('screenshot').null)

        self.assertTrue(TileSource._meta.get_field('copyrights').blank)
        self.assertEqual(TileSource._meta.get_field('copyrights').default, "")

        self.assertTrue(TileSource._meta.get_field('site').blank)
        self.assertEqual(TileSource._meta.get_field('site').default, "")

        self.assertTrue(TileSource._meta.get_field('apikey').blank)
        self.assertEqual(TileSource._meta.get_field('apikey').default, "")

        self.assertTrue(TileSource._meta.get_field('note').blank)
        self.assertEqual(TileSource._meta.get_field('note').default, "")

        self.assertTrue(TileSource._meta.get_field('slug').unique)
        self.assertTrue(TileSource._meta.get_field('slug').blank)
        self.assertTrue(TileSource._meta.get_field('slug').null)

        self.assertEqual(test_tile.upload_dir, "djeym_tile_screenshot")
        self.assertEqual(test_tile.__str__(), "OpenTopoMap")

        self.assertEqual(test_tile.title, "OpenTopoMap")
        self.assertEqual(test_tile.maxzoom, 12)
        self.assertEqual(test_tile.minzoom, 0)
        self.assertEqual(test_tile.source, "")
        self.assertEqual(bool(test_tile.screenshot), False)
        self.assertEqual(test_tile.copyrights, "")
        self.assertEqual(test_tile.site, "")
        self.assertEqual(test_tile.apikey, "")
        self.assertEqual(test_tile.note, "")
        self.assertEqual(test_tile.slug, "opentopomap")
        self.assertEqual(test_tile.admin_thumbnail(), "")

    def test_map_controls(self):
        """
        Test the model of the MapControls with the default settings.
        Тестирование модели MapControls с настройками по умолчанию.
        """
        test_map_controls = MapControls.objects.create()
        result = "['geolocationControl', 'searchControl', 'routeButtonControl'," \
                 " 'trafficControl', 'typeSelector', 'fullscreenControl'," \
                 " 'zoomControl', 'rulerControl']"

        self.assertEqual(MapControls._meta.get_field(
            'ymap').get_internal_type(), "OneToOneField")
        self.assertFalse(MapControls._meta.get_field('ymap').blank)
        self.assertTrue(MapControls._meta.get_field('ymap').null)

        self.assertFalse(MapControls._meta.get_field('geolocation').blank)
        self.assertTrue(MapControls._meta.get_field('geolocation').default)

        self.assertFalse(MapControls._meta.get_field('search').blank)
        self.assertTrue(MapControls._meta.get_field('search').default)

        self.assertFalse(MapControls._meta.get_field('provider').blank)
        self.assertTrue(MapControls._meta.get_field('provider').default)

        self.assertFalse(MapControls._meta.get_field('route').blank)
        self.assertTrue(MapControls._meta.get_field('route').default)

        self.assertFalse(MapControls._meta.get_field('traffic').blank)
        self.assertTrue(MapControls._meta.get_field('traffic').default)

        self.assertFalse(MapControls._meta.get_field('typeselector').blank)
        self.assertTrue(MapControls._meta.get_field('typeselector').default)

        self.assertFalse(MapControls._meta.get_field('fullscreen').blank)
        self.assertTrue(MapControls._meta.get_field('fullscreen').default)

        self.assertFalse(MapControls._meta.get_field('zoom').blank)
        self.assertTrue(MapControls._meta.get_field('zoom').default)

        self.assertFalse(MapControls._meta.get_field('ruler').blank)
        self.assertTrue(MapControls._meta.get_field('ruler').default)

        self.assertEqual(MapControls._meta.get_field(
            'maptype').get_internal_type(), 'CharField')
        self.assertFalse(MapControls._meta.get_field('maptype').blank)
        self.assertEqual(MapControls._meta.get_field(
            'maptype').default, 'yandex#map')
        self.assertEqual(test_map_controls.maptype, 'yandex#map')

        self.assertEqual(test_map_controls.get_control_list(), result)

    def test_external_modules(self):
        """
        Test the model of the ExternalModules with the default settings.
        Тестирование модели ExternalModules с настройками по умолчанию.
        """
        test_external_modules = ExternalModules.objects.create()

        self.assertEqual(ExternalModules._meta.get_field(
            'ymap').get_internal_type(), "OneToOneField")
        self.assertFalse(ExternalModules._meta.get_field('ymap').blank)
        self.assertTrue(ExternalModules._meta.get_field('ymap').null)

        self.assertEqual(ExternalModules._meta.get_field(
            'roundtheme').get_internal_type(), 'BooleanField')
        self.assertFalse(ExternalModules._meta.get_field('roundtheme').blank)
        self.assertFalse(ExternalModules._meta.get_field('roundtheme').default)
        self.assertFalse(test_external_modules.roundtheme)

        self.assertEqual(ExternalModules._meta.get_field(
            'heatmap').get_internal_type(), 'BooleanField')
        self.assertFalse(ExternalModules._meta.get_field('heatmap').blank)
        self.assertFalse(ExternalModules._meta.get_field('heatmap').default)
        self.assertFalse(test_external_modules.heatmap)

        self.assertEqual(ExternalModules._meta.get_field(
            'areacalculation').get_internal_type(), 'BooleanField')
        self.assertFalse(ExternalModules._meta.get_field(
            'areacalculation').blank)
        self.assertTrue(ExternalModules._meta.get_field(
            'areacalculation').default)
        self.assertTrue(test_external_modules.areacalculation)

    def test_heatmap_settings(self):
        """
        Test the model of the HeatmapSettings with the default settings.
        Тестирование модели HeatmapSettings с настройками по умолчанию.
        """
        test_heatmap_settings = HeatmapSettings.objects.create()

        self.assertEqual(HeatmapSettings._meta.get_field(
            'ymap').get_internal_type(), 'OneToOneField')
        self.assertFalse(HeatmapSettings._meta.get_field('ymap').blank)
        self.assertTrue(HeatmapSettings._meta.get_field('ymap').null)

        self.assertEqual(HeatmapSettings._meta.get_field(
            'radius').get_internal_type(), 'PositiveIntegerField')
        self.assertFalse(HeatmapSettings._meta.get_field('radius').blank)
        self.assertEqual(HeatmapSettings._meta.get_field('radius').default, 10)
        self.assertEqual(test_heatmap_settings.radius, 10)

        self.assertEqual(HeatmapSettings._meta.get_field(
            'dissipating').get_internal_type(), 'BooleanField')
        self.assertFalse(HeatmapSettings._meta.get_field('dissipating').blank)
        self.assertFalse(
            HeatmapSettings._meta.get_field('dissipating').default)
        self.assertFalse(test_heatmap_settings.dissipating)

        self.assertEqual(HeatmapSettings._meta.get_field(
            'opacity').get_internal_type(), 'CharField')
        self.assertFalse(HeatmapSettings._meta.get_field('opacity').blank)
        self.assertEqual(HeatmapSettings._meta.get_field(
            'opacity').default, '0.8')
        self.assertEqual(test_heatmap_settings.opacity, '0.8')

        self.assertEqual(HeatmapSettings._meta.get_field(
            'intensity').get_internal_type(), 'CharField')
        self.assertFalse(HeatmapSettings._meta.get_field('intensity').blank)
        self.assertEqual(HeatmapSettings._meta.get_field(
            'intensity').default, '0.2')
        self.assertEqual(test_heatmap_settings.intensity, '0.2')

        self.assertEqual(HeatmapSettings._meta.get_field(
            'gradient_color1').get_internal_type(), 'CharField')
        self.assertFalse(HeatmapSettings._meta.get_field(
            'gradient_color1').blank)
        self.assertEqual(HeatmapSettings._meta.get_field(
            'gradient_color1').default, '#56db40b3')
        self.assertEqual(test_heatmap_settings.gradient_color1, '#56db40b3')

        self.assertEqual(HeatmapSettings._meta.get_field(
            'gradient_color2').get_internal_type(), 'CharField')
        self.assertFalse(HeatmapSettings._meta.get_field(
            'gradient_color2').blank)
        self.assertEqual(HeatmapSettings._meta.get_field(
            'gradient_color2').default, '#ffd21ecc')
        self.assertEqual(test_heatmap_settings.gradient_color2, '#ffd21ecc')

        self.assertEqual(HeatmapSettings._meta.get_field(
            'gradient_color3').get_internal_type(), 'CharField')
        self.assertFalse(HeatmapSettings._meta.get_field(
            'gradient_color3').blank)
        self.assertEqual(HeatmapSettings._meta.get_field(
            'gradient_color3').default, '#ed4543e6')
        self.assertEqual(test_heatmap_settings.gradient_color3, '#ed4543e6')

        self.assertEqual(HeatmapSettings._meta.get_field(
            'gradient_color4').get_internal_type(), 'CharField')
        self.assertFalse(HeatmapSettings._meta.get_field(
            'gradient_color4').blank)
        self.assertEqual(HeatmapSettings._meta.get_field(
            'gradient_color4').default, '#b22222')
        self.assertEqual(test_heatmap_settings.gradient_color4, '#b22222')

        self.assertIn(test_heatmap_settings.__str__(),
                      ('Settings', 'Настройки'))

    def test_heat_point(self):
        """
        Test the model of the HeatPoint with the default settings.
        Тестирование модели HeatPoint с настройками по умолчанию.
        """
        title_map = self.title_map
        text_json_1 = '{"type": "Feature", "id": 0, "geometry": {"type": "Point", ' \
                      '"coordinates": []}, "properties": {"weight": 0}}'
        text_json_2 = '{"type": "Feature", "id": 1, "geometry": {"type": "Point", ' \
                      '"coordinates": [0, 0]}, "properties": {"weight": 0}}'
        test_map = Map.objects.create(title=title_map)
        test_heat_point = HeatPoint.objects.create(ymap=test_map)

        self.assertEqual(HeatPoint._meta.get_field(
            'ymap').get_internal_type(), 'ForeignKey')
        self.assertFalse(HeatPoint._meta.get_field('ymap').blank)
        self.assertFalse(HeatPoint._meta.get_field('ymap').null)

        self.assertEqual(HeatPoint._meta.get_field(
            'title').get_internal_type(), 'CharField')
        self.assertTrue(HeatPoint._meta.get_field(
            'title').blank)
        self.assertEqual(HeatPoint._meta.get_field(
            'title').default, "")
        self.assertIn(test_heat_point.title,
                      ('Heat Point - 1', 'Тепловая точка - 1'))

        self.assertEqual(HeatPoint._meta.get_field(
            'weight').get_internal_type(), 'PositiveIntegerField')
        self.assertTrue(HeatPoint._meta.get_field(
            'weight').blank)
        self.assertEqual(HeatPoint._meta.get_field(
            'weight').default, 0)
        self.assertEqual(test_heat_point.weight, 0)

        self.assertEqual(HeatPoint._meta.get_field(
            'coordinates').get_internal_type(), 'CharField')
        self.assertFalse(HeatPoint._meta.get_field(
            'coordinates').blank)
        self.assertEqual(HeatPoint._meta.get_field(
            'coordinates').default, '[0,0]')
        self.assertEqual(test_heat_point.coordinates, '[0,0]')

        self.assertEqual(HeatPoint._meta.get_field(
            'active').get_internal_type(), 'BooleanField')
        self.assertFalse(HeatPoint._meta.get_field(
            'active').blank)
        self.assertTrue(HeatPoint._meta.get_field(
            'active').default)
        self.assertTrue(test_heat_point.active)

        self.assertEqual(HeatPoint._meta.get_field(
            'json_code').get_internal_type(), 'TextField')
        self.assertTrue(HeatPoint._meta.get_field(
            'json_code').blank)
        self.assertEqual(HeatPoint._meta.get_field(
            'json_code').default, text_json_1)
        self.assertEqual(test_heat_point.json_code, text_json_2)

        self.assertIn(test_heat_point.__str__(),
                      ('Heat Point - 1', 'Тепловая точка - 1'))

    def test_preset(self):
        """
        Test the model of the Preset with the default settings.
        Тестирование модели Preset с настройками по умолчанию.
        """
        title_map = self.title_map
        title = 'iPreset'
        ymap = Map.objects.create(title=title_map)
        preset = Preset.objects.create(ymap=ymap, title=title)

        self.assertEqual(Preset._meta.get_field(
            'title').get_internal_type(), 'CharField')
        self.assertEqual(Preset._meta.get_field(
            'title').max_length, 60)
        self.assertFalse(Preset._meta.get_field(
            'title').unique)
        self.assertFalse(Preset._meta.get_field(
            'title').blank)
        self.assertEqual(Preset._meta.get_field(
            'title').default, "")
        self.assertEqual(preset.title, title)

        self.assertEqual(Preset._meta.get_field(
            'icon').get_internal_type(), 'CharField')
        self.assertEqual(Preset._meta.get_field(
            'icon').max_length, 255)
        self.assertFalse(Preset._meta.get_field(
            'icon').unique)
        self.assertTrue(Preset._meta.get_field(
            'icon').blank)
        self.assertEqual(Preset._meta.get_field(
            'icon').default, '<i class="far fa-smile"></i>')
        self.assertEqual(preset.icon, '<i class="far fa-smile"></i>')

        self.assertEqual(Preset._meta.get_field(
            'html').get_internal_type(), 'TextField')
        self.assertFalse(Preset._meta.get_field(
            'html').unique)
        self.assertFalse(Preset._meta.get_field(
            'html').blank)
        self.assertEqual(Preset._meta.get_field(
            'html').default, '<p style="color:#e91e63;">iPreset</p>')
        self.assertEqual(preset.html, '<p style="color:#e91e63;">iPreset</p>')

        self.assertEqual(Preset._meta.get_field(
            'js').get_internal_type(), 'TextField')
        self.assertFalse(Preset._meta.get_field(
            'js').unique)
        self.assertTrue(Preset._meta.get_field(
            'js').blank)
        self.assertEqual(Preset._meta.get_field(
            'js').default, "")
        self.assertEqual(preset.js, "")

        self.assertEqual(Preset._meta.get_field(
            'description').get_internal_type(), 'TextField')
        self.assertFalse(Preset._meta.get_field(
            'description').unique)
        self.assertFalse(Preset._meta.get_field(
            'description').blank)
        self.assertEqual(Preset._meta.get_field(
            'description').default, """<div>Описание</div>
<div style="color:#607D8B;">Description</div>""")
        self.assertEqual(preset.description, """<div>Описание</div>
<div style="color:#607D8B;">Description</div>""")

        self.assertEqual(Preset._meta.get_field(
            'autoheader').get_internal_type(), 'BooleanField')
        self.assertFalse(Preset._meta.get_field(
            'autoheader').unique)
        self.assertFalse(Preset._meta.get_field(
            'autoheader').blank)
        self.assertFalse(Preset._meta.get_field(
            'autoheader').default)
        self.assertFalse(preset.autoheader)

        self.assertEqual(Preset._meta.get_field(
            'autobody').get_internal_type(), 'BooleanField')
        self.assertFalse(Preset._meta.get_field(
            'autobody').unique)
        self.assertFalse(Preset._meta.get_field(
            'autobody').blank)
        self.assertFalse(Preset._meta.get_field(
            'autobody').default)
        self.assertFalse(preset.autobody)

        self.assertEqual(Preset._meta.get_field(
            'autofooter').get_internal_type(), 'BooleanField')
        self.assertFalse(Preset._meta.get_field(
            'autofooter').unique)
        self.assertFalse(Preset._meta.get_field(
            'autofooter').blank)
        self.assertFalse(Preset._meta.get_field(
            'autofooter').default)
        self.assertFalse(preset.autofooter)

        self.assertEqual(Preset._meta.get_field(
            'placemark').get_internal_type(), 'BooleanField')
        self.assertFalse(Preset._meta.get_field(
            'placemark').unique)
        self.assertFalse(Preset._meta.get_field(
            'placemark').blank)
        self.assertTrue(Preset._meta.get_field(
            'placemark').default)
        self.assertTrue(preset.placemark)

        self.assertEqual(Preset._meta.get_field(
            'polyline').get_internal_type(), 'BooleanField')
        self.assertFalse(Preset._meta.get_field(
            'polyline').unique)
        self.assertFalse(Preset._meta.get_field(
            'polyline').blank)
        self.assertTrue(Preset._meta.get_field(
            'polyline').default)
        self.assertTrue(preset.polyline)

        self.assertEqual(Preset._meta.get_field(
            'polygon').get_internal_type(), 'BooleanField')
        self.assertFalse(Preset._meta.get_field(
            'polygon').unique)
        self.assertFalse(Preset._meta.get_field(
            'polygon').blank)
        self.assertTrue(Preset._meta.get_field(
            'polygon').default)
        self.assertTrue(preset.polygon)

        self.assertEqual(Preset._meta.get_field(
            'position').get_internal_type(), 'PositiveSmallIntegerField')
        self.assertFalse(Preset._meta.get_field(
            'position').unique)
        self.assertTrue(Preset._meta.get_field(
            'position').blank)
        self.assertEqual(Preset._meta.get_field(
            'position').default, 0)
        self.assertEqual(preset.position, 0)

    def test_general_settings(self):
        """
        Test the model of the GeneralSettings with the default settings.
        Тестирование модели GeneralSettings с настройками по умолчанию.
        """
        title_map = self.title_map
        ymap = Map.objects.create(title=title_map)
        general_settings = ymap.general_settings

        self.assertEqual(GeneralSettings._meta.get_field(
            'ymap').get_internal_type(), 'OneToOneField')
        self.assertFalse(GeneralSettings._meta.get_field('ymap').blank)
        self.assertTrue(GeneralSettings._meta.get_field('ymap').null)

        self.assertEqual(GeneralSettings._meta.get_field(
            'clustering_edit').get_internal_type(), 'BooleanField')
        self.assertFalse(GeneralSettings._meta.get_field(
            'clustering_edit').unique)
        self.assertFalse(GeneralSettings._meta.get_field(
            'clustering_edit').blank)
        self.assertTrue(GeneralSettings._meta.get_field(
            'clustering_edit').default)
        self.assertTrue(general_settings.clustering_edit)

        self.assertEqual(GeneralSettings._meta.get_field(
            'clustering_site').get_internal_type(), 'BooleanField')
        self.assertFalse(GeneralSettings._meta.get_field(
            'clustering_site').unique)
        self.assertFalse(GeneralSettings._meta.get_field(
            'clustering_site').blank)
        self.assertTrue(GeneralSettings._meta.get_field(
            'clustering_site').default)
        self.assertTrue(general_settings.clustering_site)

        self.assertEqual(GeneralSettings._meta.get_field(
            'cluster_layout').get_internal_type(), 'CharField')
        self.assertEqual(GeneralSettings._meta.get_field(
            'cluster_layout').max_length, 255)
        self.assertFalse(GeneralSettings._meta.get_field(
            'cluster_layout').unique)
        self.assertFalse(GeneralSettings._meta.get_field(
            'cluster_layout').blank)
        self.assertEqual(GeneralSettings._meta.get_field(
            'cluster_layout').default, "cluster#balloonTwoColumns")
        self.assertEqual(general_settings.cluster_layout,
                         "cluster#balloonTwoColumns")

        self.assertEqual(GeneralSettings._meta.get_field(
            'cluster_icon_content').get_internal_type(), 'BooleanField')
        self.assertFalse(GeneralSettings._meta.get_field(
            'cluster_icon_content').unique)
        self.assertFalse(GeneralSettings._meta.get_field(
            'cluster_icon_content').blank)
        self.assertTrue(GeneralSettings._meta.get_field(
            'cluster_icon_content').default)
        self.assertTrue(general_settings.cluster_icon_content)

        self.assertEqual(GeneralSettings._meta.get_field(
            'cluster_icon_content_bg_color').get_internal_type(), 'CharField')
        self.assertEqual(GeneralSettings._meta.get_field(
            'cluster_icon_content_bg_color').max_length, 255)
        self.assertFalse(GeneralSettings._meta.get_field(
            'cluster_icon_content_bg_color').unique)
        self.assertFalse(GeneralSettings._meta.get_field(
            'cluster_icon_content_bg_color').blank)
        self.assertEqual(GeneralSettings._meta.get_field(
            'cluster_icon_content_bg_color').default, "#ffffff")
        self.assertEqual(
            general_settings.cluster_icon_content_bg_color, "#ffffff")

        self.assertEqual(GeneralSettings._meta.get_field(
            'cluster_icon_content_txt_color').get_internal_type(), 'CharField')
        self.assertEqual(GeneralSettings._meta.get_field(
            'cluster_icon_content_txt_color').max_length, 255)
        self.assertFalse(GeneralSettings._meta.get_field(
            'cluster_icon_content_txt_color').unique)
        self.assertFalse(GeneralSettings._meta.get_field(
            'cluster_icon_content_txt_color').blank)
        self.assertEqual(GeneralSettings._meta.get_field(
            'cluster_icon_content_txt_color').default, "#333333")
        self.assertEqual(
            general_settings.cluster_icon_content_txt_color, "#333333")

        self.assertEqual(GeneralSettings._meta.get_field(
            'disable_site_panel').get_internal_type(), 'BooleanField')
        self.assertFalse(GeneralSettings._meta.get_field(
            'disable_site_panel').unique)
        self.assertFalse(GeneralSettings._meta.get_field(
            'disable_site_panel').blank)
        self.assertFalse(GeneralSettings._meta.get_field(
            'disable_site_panel').default)
        self.assertFalse(general_settings.disable_site_panel)

    def test_map(self):
        """
        Test the model of the Map with the default settings.
        Тестирование модели Map с настройками по умолчанию.
        """
        title_map = self.title_map
        test_map = Map.objects.create(title=title_map)

        self.assertTrue(Map._meta.get_field('title').unique)
        self.assertFalse(Map._meta.get_field('title').blank)

        self.assertFalse(Map._meta.get_field('icon_cluster').blank)
        self.assertTrue(Map._meta.get_field('icon_cluster').null)
        self.assertEqual(Map._meta.get_field('icon_cluster')
                         .get_internal_type(), "ForeignKey")

        self.assertFalse(Map._meta.get_field('icon_collection').blank)
        self.assertTrue(Map._meta.get_field('icon_collection').null)
        self.assertEqual(Map._meta.get_field('icon_collection')
                         .get_internal_type(), "ForeignKey")

        self.assertTrue(Map._meta.get_field('tile').blank)
        self.assertTrue(Map._meta.get_field('tile').null)
        self.assertEqual(Map._meta.get_field('tile')
                         .get_internal_type(), "ForeignKey")

        self.assertTrue(hasattr(test_map, 'controls'))

        self.assertEqual(test_map.title, title_map)
        self.assertEqual(test_map.latitude, "0")
        self.assertEqual(test_map.longitude, "0")
        self.assertFalse(bool(test_map.icon_collection))
        self.assertTrue(test_map.active)
        self.assertEqual(test_map.zoom, 3)
        self.assertEqual(test_map.slug, "supermarkety-kharkova")

        self.assertEqual(test_map.__str__(), title_map)

        self.assertEqual(Map._meta.ordering, ('title',))

        self.assertEqual(test_map.get_absolute_url(),
                         "/djeym/ymeditor/supermarkety-kharkova/")

        self.assertEqual(test_map.get_custom_cluster(), "")
        self.assertEqual(test_map.get_custom_marker_icon(), "")
        self.assertEqual(test_map.get_tile_screenshot(
        ), '<img src="/static/djeym/img/default_tile.png" height="60" alt="Screenshot">')

    def test_category_placemark(self):
        """
        Test the model of the CategoryPlacemark with the default settings.
        Тестирование модели CategoryPlacemark с настройками по умолчанию.
        """
        title_map = self.title_map
        title_category_placemark_1 = self.title_category_placemark_1
        title_category_placemark_2 = self.title_category_placemark_2

        test_map = Map.objects.create(title=title_map)
        category_1 = CategoryPlacemark.objects.create(
            ymap=test_map,
            title=title_category_placemark_1)
        category_2 = CategoryPlacemark.objects.create(
            ymap=test_map,
            title=title_category_placemark_2)

        self.assertEqual(test_map.category_placemark.count(), 2)
        self.assertEqual(CategoryPlacemark._meta.get_field('ymap')
                         .get_internal_type(), "ForeignKey")

        self.assertFalse(CategoryPlacemark._meta.get_field('title').unique)
        self.assertFalse(CategoryPlacemark._meta.get_field('title').blank)

        self.assertEqual(category_1.title, title_category_placemark_1)
        self.assertTrue(category_1.active)
        self.assertEqual(category_1.ymap.slug, test_map.slug)
        self.assertEqual(category_1.get_map_name(), test_map.title)
        self.assertEqual(category_1.get_title(), title_category_placemark_1)
        self.assertEqual(category_1.category_icon, "")
        self.assertEqual(category_1.category_color, "#00bfff")

        self.assertEqual(category_1.get_category_icon(), "")

        self.assertEqual(
            category_1.get_category_color(),
            "<div class=\"djeym_category_color\" style=\"background:#00bfff;\"></div>")

        self.assertEqual(category_2.title, title_category_placemark_2)
        self.assertTrue(category_2.active)
        self.assertEqual(category_2.ymap.slug, test_map.slug)
        self.assertEqual(category_2.get_map_name(), test_map.title)
        self.assertEqual(category_2.get_title(), title_category_placemark_2)
        self.assertEqual(category_2.category_icon, "")
        self.assertEqual(category_2.category_color, "#00bfff")

        self.assertEqual(category_2.get_category_icon(), "")

        self.assertEqual(
            category_2.get_category_color(),
            "<div class=\"djeym_category_color\" style=\"background:#00bfff;\"></div>")

        self.assertEqual(CategoryPlacemark._meta.ordering, ('title',))

    def test_subcategory_placemark(self):
        """
        Test the model of the SubCategoryPlacemark with the default settings.
        Тестирование модели SubCategoryPlacemark с настройками по умолчанию.
        """
        title_map = self.title_map
        title_subcategory_placemark_1 = self.title_subcategory_placemark_1
        title_subcategory_placemark_2 = self.title_subcategory_placemark_2

        test_map = Map.objects.create(title=title_map)
        subcategory_1 = SubCategoryPlacemark.objects.create(
            ymap=test_map,
            title=title_subcategory_placemark_1)
        subcategory_2 = SubCategoryPlacemark.objects.create(
            ymap=test_map,
            title=title_subcategory_placemark_2)

        self.assertEqual(test_map.subcategory_placemark.all().count(), 2)
        self.assertEqual(SubCategoryPlacemark._meta.get_field('ymap')
                         .get_internal_type(), "ForeignKey")

        self.assertFalse(SubCategoryPlacemark._meta.get_field('title').unique)
        self.assertFalse(SubCategoryPlacemark._meta.get_field('title').blank)

        self.assertEqual(subcategory_1.title, title_subcategory_placemark_1)
        self.assertEqual(subcategory_1.category_icon, "")
        self.assertEqual(subcategory_1.category_color, "#ffcc00")
        self.assertTrue(subcategory_1.active)
        self.assertEqual(subcategory_1.ymap.slug, test_map.slug)
        self.assertEqual(subcategory_1.get_map_name(), test_map.title)
        self.assertEqual(subcategory_1.get_title(),
                         title_subcategory_placemark_1)

        self.assertEqual(subcategory_1.get_category_icon(), "")

        self.assertEqual(
            subcategory_1.get_category_color(),
            "<div class=\"djeym_category_color\" style=\"background:#ffcc00;\"></div>")

        self.assertEqual(subcategory_2.title, title_subcategory_placemark_2)
        self.assertEqual(subcategory_2.category_icon, "")
        self.assertEqual(subcategory_2.category_color, "#ffcc00")
        self.assertTrue(subcategory_2.active)
        self.assertEqual(subcategory_2.ymap.slug, test_map.slug)
        self.assertEqual(subcategory_2.get_map_name(), test_map.title)
        self.assertEqual(subcategory_2.get_title(),
                         title_subcategory_placemark_2)

        self.assertEqual(subcategory_2.get_category_icon(), "")

        self.assertEqual(
            subcategory_2.get_category_color(),
            "<div class=\"djeym_category_color\" style=\"background:#ffcc00;\"></div>")

        self.assertEqual(SubCategoryPlacemark._meta.ordering, ('title',))

    def test_category_polyline(self):
        """
        Test the model of the CategoryPolyline with the default settings.
        Тестирование модели CategoryPolyline с настройками по умолчанию.
        """
        title_map = self.title_map
        title_category_polyline_1 = self.title_category_polyline_1
        title_category_polyline_2 = self.title_category_polyline_2

        test_map = Map.objects.create(title=title_map)

        category_1 = CategoryPolyline.objects.create(
            ymap=test_map,
            title=title_category_polyline_1)
        category_2 = CategoryPolyline.objects.create(
            ymap=test_map,
            title=title_category_polyline_2)

        self.assertEqual(test_map.category_polyline.count(), 2)
        self.assertEqual(CategoryPolyline._meta.get_field('ymap')
                         .get_internal_type(), "ForeignKey")

        self.assertFalse(CategoryPolyline._meta.get_field('title').unique)
        self.assertFalse(CategoryPolyline._meta.get_field('title').blank)

        self.assertEqual(category_1.title, title_category_polyline_1)
        self.assertEqual(category_1.category_icon, "")
        self.assertEqual(category_1.category_color, "#00bfff")
        self.assertTrue(category_1.active)
        self.assertEqual(category_1.ymap.slug, test_map.slug)
        self.assertEqual(category_1.get_map_name(), test_map.title)
        self.assertEqual(category_1.get_title(), title_category_polyline_1)

        self.assertEqual(category_1.get_category_icon(), "")

        self.assertEqual(
            category_1.get_category_color(),
            "<div class=\"djeym_category_color\" style=\"background:#00bfff;\"></div>")

        self.assertEqual(category_2.title, title_category_polyline_2)
        self.assertEqual(category_2.category_icon, "")
        self.assertEqual(category_2.category_color, "#00bfff")
        self.assertTrue(category_2.active)
        self.assertEqual(category_2.ymap.slug, test_map.slug)
        self.assertEqual(category_2.get_map_name(), test_map.title)
        self.assertEqual(category_2.get_title(), title_category_polyline_2)

        self.assertEqual(category_2.get_category_icon(), "")

        self.assertEqual(
            category_2.get_category_color(),
            "<div class=\"djeym_category_color\" style=\"background:#00bfff;\"></div>")

        self.assertEqual(CategoryPolyline._meta.ordering, ('title',))

    def test_category_polygon(self):
        """
        Test the model of the CategoryPolygon with the default settings.
        Тестирование модели CategoryPolygon с настройками по умолчанию.
        """
        title_map = self.title_map
        title_category_polygon_1 = self.title_category_polygon_1
        title_category_polygon_2 = self.title_category_polygon_2

        test_map = Map.objects.create(title=title_map)

        category_1 = CategoryPolygon.objects.create(
            ymap=test_map,
            title=title_category_polygon_1)
        category_2 = CategoryPolygon.objects.create(
            ymap=test_map,
            title=title_category_polygon_2)

        self.assertEqual(test_map.category_polygon.count(), 2)
        self.assertEqual(CategoryPolygon._meta.get_field('ymap')
                         .get_internal_type(), "ForeignKey")

        self.assertFalse(CategoryPolygon._meta.get_field('title').unique)
        self.assertFalse(CategoryPolygon._meta.get_field('title').blank)

        self.assertEqual(category_1.title, title_category_polygon_1)
        self.assertEqual(category_1.category_icon, "")
        self.assertEqual(category_1.category_color, "#00bfff")
        self.assertTrue(category_1.active)
        self.assertEqual(category_1.ymap.slug, test_map.slug)
        self.assertEqual(category_1.get_map_name(), test_map.title)
        self.assertEqual(category_1.get_title(), title_category_polygon_1)

        self.assertEqual(category_1.get_category_icon(), "")

        self.assertEqual(
            category_1.get_category_color(),
            "<div class=\"djeym_category_color\" style=\"background:#00bfff;\"></div>")

        self.assertEqual(category_2.title, title_category_polygon_2)
        self.assertEqual(category_2.category_icon, "")
        self.assertEqual(category_2.category_color, "#00bfff")
        self.assertTrue(category_2.active)
        self.assertEqual(category_2.ymap.slug, test_map.slug)
        self.assertEqual(category_2.get_map_name(), test_map.title)
        self.assertEqual(category_2.get_title(), title_category_polygon_2)

        self.assertEqual(category_1.get_category_icon(), "")

        self.assertEqual(
            category_1.get_category_color(),
            "<div class=\"djeym_category_color\" style=\"background:#00bfff;\"></div>")

        self.assertEqual(CategoryPolygon._meta.ordering, ('title',))

    def test_placemark(self):
        """
        Test the model of the Placemark with the default settings.
        Тестирование модели Placemark с настройками по умолчанию.
        """
        title_map = self.title_map
        title_category_placemark_1 = self.title_category_placemark_1
        title_subcategory_placemark_1 = self.title_subcategory_placemark_1
        title_subcategory_placemark_2 = self.title_subcategory_placemark_2
        header_placemark_1 = self.header_placemark_1
        header_placemark_2 = self.header_placemark_2
        custom_marker_icon = self.custom_marker_icon_1
        title_icon_collection = self.title_icon_collection

        collection = IconCollection.objects.create(title=title_icon_collection)

        icon_marker = MarkerIcon.objects.create(
            icon_collection=collection,
            title=custom_marker_icon,
            svg="test_marker.svg")

        test_map = Map.objects.create(title=title_map)
        category = CategoryPlacemark.objects.create(
            ymap=test_map,
            title=title_category_placemark_1)
        subcategory_1 = SubCategoryPlacemark.objects.create(
            ymap=test_map,
            title=title_subcategory_placemark_1)
        subcategory_2 = SubCategoryPlacemark.objects.create(
            ymap=test_map,
            title=title_subcategory_placemark_2)
        placemark_1 = Placemark.objects.create(
            ymap=test_map,
            category=category,
            header=header_placemark_1,
            icon_name=icon_marker.slug)
        placemark_1.subcategories.add(subcategory_1, subcategory_2)
        placemark_2 = Placemark.objects.create(
            ymap=test_map,
            category=category,
            header=header_placemark_2,
            icon_name=icon_marker.slug)
        placemark_2.subcategories.add(subcategory_1, subcategory_2)

        self.assertEqual(test_map.placemark_map.all().count(), 2)
        self.assertEqual(category.placemark_category.all().count(), 2)
        self.assertEqual(
            subcategory_1.placemark_subcategories.all().count(), 2)
        self.assertEqual(
            subcategory_2.placemark_subcategories.all().count(), 2)

        self.assertEqual(Placemark._meta.get_field('ymap')
                         .get_internal_type(), "ForeignKey")
        self.assertEqual(Placemark._meta.get_field('category')
                         .get_internal_type(), "ChainedForeignKey")
        self.assertEqual(Placemark._meta.get_field('subcategories')
                         .get_internal_type(), "ChainedManyToManyField")

        self.assertFalse(Placemark._meta.get_field('header').blank)

        self.assertEqual(placemark_1.ymap.pk, test_map.pk)
        self.assertEqual(placemark_1.category.pk, category.pk)
        self.assertEqual(placemark_1.subcategories.get(
            pk=1).pk, subcategory_1.pk)
        self.assertEqual(placemark_1.subcategories.get(
            pk=2).pk, subcategory_2.pk)
        self.assertEqual(placemark_1.header, "<p>Test placemark 1</p>")
        self.assertEqual(placemark_1.__str__(), "<p>Test placemark 1</p>")
        self.assertEqual(placemark_1.body, "")
        self.assertEqual(placemark_1.footer, "")
        self.assertEqual(placemark_1.icon_name, icon_marker.slug)
        self.assertEqual(placemark_1.coordinates, "[0,0]")
        self.assertEqual(placemark_1.like, 0)
        self.assertEqual(placemark_1.dislike, 0)
        self.assertTrue(placemark_1.active)
        self.assertEqual(placemark_1.ymap.slug, test_map.slug)

        self.assertEqual(placemark_2.ymap.pk, test_map.pk)
        self.assertEqual(placemark_2.category.pk, category.pk)
        self.assertEqual(placemark_2.subcategories.get(
            pk=1).pk, subcategory_1.pk)
        self.assertEqual(placemark_2.subcategories.get(
            pk=2).pk, subcategory_2.pk)
        self.assertEqual(placemark_2.header, "<p>Test placemark 2</p>")
        self.assertEqual(placemark_2.__str__(), "<p>Test placemark 2</p>")
        self.assertEqual(placemark_2.body, "")
        self.assertEqual(placemark_2.footer, "")
        self.assertEqual(placemark_2.icon_name, icon_marker.slug)
        self.assertEqual(placemark_2.coordinates, "[0,0]")
        self.assertEqual(placemark_2.like, 0)
        self.assertEqual(placemark_2.dislike, 0)
        self.assertTrue(placemark_2.active)
        self.assertEqual(placemark_2.ymap.slug, test_map.slug)

        self.assertEqual(Placemark._meta.ordering, ("-id",))

    def test_polyline(self):
        """
        Test the model of the Polyline with the default settings.
        Тестирование модели Polyline с настройками по умолчанию.
        """
        title_map = self.title_map
        title_category_polyline_1 = self.title_category_polyline_1
        header_polyline_1 = self.header_polyline_1
        header_polyline_2 = self.header_polyline_2
        coordinates = '[[0,0],[11,11]]'

        test_map = Map.objects.create(title=title_map)
        category = CategoryPolyline.objects.create(
            ymap=test_map,
            title=title_category_polyline_1)
        polyline_1 = Polyline.objects.create(
            ymap=test_map,
            category=category,
            header=header_polyline_1,
            coordinates=coordinates)
        polyline_2 = Polyline.objects.create(
            ymap=test_map,
            category=category,
            header=header_polyline_2,
            coordinates=coordinates)

        self.assertEqual(Polyline._meta.get_field('ymap')
                         .get_internal_type(), "ForeignKey")
        self.assertEqual(Polyline._meta.get_field('category')
                         .get_internal_type(), "ChainedForeignKey")

        self.assertEqual(test_map.polyline_map.all().count(), 2)
        self.assertEqual(category.polyline_category.all().count(), 2)

        self.assertFalse(Polyline._meta.get_field('header').blank)

        self.assertEqual(polyline_1.ymap.pk, test_map.pk)
        self.assertEqual(polyline_1.category.pk, category.pk)
        self.assertEqual(polyline_1.header, "<p>Test polyline 1</p>")
        self.assertEqual(polyline_1.__str__(), "<p>Test polyline 1</p>")
        self.assertEqual(polyline_1.body, "")
        self.assertEqual(polyline_1.footer, "")
        self.assertEqual(polyline_1.stroke_width, 5)
        self.assertEqual(polyline_1.stroke_color, "#1e98ff")
        self.assertEqual(polyline_1.stroke_opacity, '0.9')
        self.assertEqual(polyline_1.coordinates, coordinates)
        self.assertEqual(polyline_1.like, 0)
        self.assertEqual(polyline_1.dislike, 0)
        self.assertTrue(polyline_1.active)

        self.assertEqual(polyline_2.ymap.pk, test_map.pk)
        self.assertEqual(polyline_2.category.pk, category.pk)
        self.assertEqual(polyline_2.header, "<p>Test polyline 2</p>")
        self.assertEqual(polyline_2.__str__(), "<p>Test polyline 2</p>")
        self.assertEqual(polyline_2.body, "")
        self.assertEqual(polyline_2.footer, "")
        self.assertEqual(polyline_2.stroke_width, 5)
        self.assertEqual(polyline_2.stroke_color, "#1e98ff")
        self.assertEqual(polyline_2.stroke_opacity, '0.9')
        self.assertEqual(polyline_2.coordinates, coordinates)
        self.assertEqual(polyline_2.like, 0)
        self.assertEqual(polyline_2.dislike, 0)
        self.assertTrue(polyline_2.active)

        self.assertEqual(Polyline._meta.ordering, ("-id",))

    def test_polygon(self):
        """
        Test the model of the Polygon with the default settings.
        Тестирование модели Polygon с настройками по умолчанию.
        """
        title_map = self.title_map
        title_category_polygon_1 = self.title_category_polygon_1
        header_polygon_1 = self.header_polygon_1
        header_polygon_2 = self.header_polygon_2
        coordinates = '[[0,0],[11,11],[22,22],[33,33]]'

        test_map = Map.objects.create(title=title_map)
        category = CategoryPolygon.objects.create(
            ymap=test_map,
            title=title_category_polygon_1)
        polygon_1 = Polygon.objects.create(
            ymap=test_map,
            category=category,
            header=header_polygon_1,
            coordinates=coordinates)
        polygon_2 = Polygon.objects.create(
            ymap=test_map,
            category=category,
            header=header_polygon_2,
            coordinates=coordinates)

        self.assertEqual(Polygon._meta.get_field('ymap')
                         .get_internal_type(), "ForeignKey")
        self.assertEqual(Polygon._meta.get_field('category')
                         .get_internal_type(), "ChainedForeignKey")

        self.assertEqual(test_map.polygon_map.all().count(), 2)
        self.assertEqual(category.polygon_category.all().count(), 2)

        self.assertFalse(Polygon._meta.get_field('header').blank)

        self.assertEqual(polygon_1.ymap.pk, test_map.pk)
        self.assertEqual(polygon_1.category.pk, category.pk)
        self.assertEqual(polygon_1.header, "<p>Test polygon 1</p>")
        self.assertEqual(polygon_1.__str__(), "<p>Test polygon 1</p>")
        self.assertEqual(polygon_1.body, "")
        self.assertEqual(polygon_1.footer, "")
        self.assertEqual(polygon_1.stroke_width, 2)
        self.assertEqual(polygon_1.stroke_color, "#1e98ff")
        self.assertEqual(polygon_1.stroke_opacity, '0.9')
        self.assertEqual(polygon_1.fill_color, "#1e98ff")
        self.assertEqual(polygon_1.fill_opacity, '0.9')
        self.assertEqual(polygon_1.coordinates, coordinates)
        self.assertEqual(polygon_1.like, 0)
        self.assertEqual(polygon_1.dislike, 0)
        self.assertTrue(polygon_1.active)

        self.assertEqual(polygon_2.ymap.pk, test_map.pk)
        self.assertEqual(polygon_2.category.pk, category.pk)
        self.assertEqual(polygon_2.header, "<p>Test polygon 2</p>")
        self.assertEqual(polygon_2.__str__(), "<p>Test polygon 2</p>")
        self.assertEqual(polygon_2.body, "")
        self.assertEqual(polygon_2.footer, "")
        self.assertEqual(polygon_2.stroke_width, 2)
        self.assertEqual(polygon_2.stroke_color, "#1e98ff")
        self.assertEqual(polygon_2.stroke_opacity, '0.9')
        self.assertEqual(polygon_2.fill_color, "#1e98ff")
        self.assertEqual(polygon_2.fill_opacity, '0.9')
        self.assertEqual(polygon_2.coordinates, coordinates)
        self.assertEqual(polygon_2.like, 0)
        self.assertEqual(polygon_2.dislike, 0)
        self.assertTrue(polygon_2.active)

        self.assertEqual(Polygon._meta.ordering, ("-id",))

    def test_cluster_icon_check_without_image(self):
        """
        Test the model of the ClusterIcon with the default settings.
        Тестирование модели ClusterIcon с настройками по умолчанию.
        """
        title_cluster_icon = self.title_cluster_icon

        cluster_icon = ClusterIcon.objects.create(
            title=title_cluster_icon)

        self.assertTrue(ClusterIcon._meta.get_field('title').unique)
        self.assertFalse(ClusterIcon._meta.get_field('title').blank)
        self.assertFalse(ClusterIcon._meta.get_field('svg').blank)

        self.assertEqual(cluster_icon.title, "Test Cluster Icon")
        self.assertFalse(bool(cluster_icon.svg))
        self.assertEqual(cluster_icon.size_width, 0)
        self.assertEqual(cluster_icon.size_height, 0)
        self.assertEqual(cluster_icon.offset_x, Decimal(".0"))
        self.assertEqual(cluster_icon.offset_y, Decimal(".0"))
        self.assertEqual(cluster_icon.slug, "test-cluster-icon")

        self.assertEqual(cluster_icon.upload_dir, "djeym_custom_icons")
        self.assertEqual(cluster_icon.__str__(), "Test Cluster Icon")
        self.assertEqual(cluster_icon.admin_thumbnail(), "")

        self.assertEqual(ClusterIcon._meta.ordering, ("title", "id"))

        self.assertEqual(cluster_icon.get_size(), "[0,0]")
        self.assertEqual(cluster_icon.get_offset(), "[0.0,0.0]")

    def test_cluster_icon_check_with_image(self):
        """
        Test the model of the ClusterIcon with the default settings.
        Тестирование модели ClusterIcon с настройками по умолчанию.
        """
        title_cluster_icon = self.title_cluster_icon

        cluster_icon = ClusterIcon.objects.create(
            title=title_cluster_icon, svg="test_cluster.svg")

        self.assertTrue(ClusterIcon._meta.get_field('title').unique)
        self.assertFalse(ClusterIcon._meta.get_field('title').blank)
        self.assertFalse(ClusterIcon._meta.get_field('svg').blank)

        self.assertEqual(cluster_icon.title, "Test Cluster Icon")
        self.assertTrue(bool(cluster_icon.svg))
        self.assertEqual(cluster_icon.size_width, 60)
        self.assertEqual(cluster_icon.size_height, 60)
        self.assertEqual(cluster_icon.offset_x, Decimal("-30.0"))
        self.assertEqual(cluster_icon.offset_y, Decimal("-30.0"))
        self.assertEqual(cluster_icon.slug, "test-cluster-icon")

        self.assertEqual(cluster_icon.upload_dir, "djeym_custom_icons")
        self.assertEqual(cluster_icon.__str__(), "Test Cluster Icon")

        self.assertEqual(
            cluster_icon.admin_thumbnail(),
            "<img src=\"/media/test_cluster.svg\" height=\"60\" alt=\"Icon\">")

        self.assertEqual(ClusterIcon._meta.ordering, ("title", "id"))

        self.assertEqual(cluster_icon.get_size(), "[60,60]")
        self.assertEqual(cluster_icon.get_offset(), "[-30.0,-30.0]")

    def test_icon_collection(self):
        """
        Test the model of the IconCollection with the default settings.
        Тестирование модели IconCollection с настройками по умолчанию.
        """
        title_icon_collection = self.title_icon_collection

        collection = IconCollection.objects.create(title=title_icon_collection)

        self.assertTrue(IconCollection._meta.get_field('title').unique)
        self.assertFalse(IconCollection._meta.get_field('title').blank)
        self.assertEqual(IconCollection._meta.get_field(
            'title').max_length, 60)

        self.assertEqual(collection.__str__(), title_icon_collection)
        self.assertEqual(collection.admin_thumbnail(), "")
        self.assertEqual(IconCollection._meta.ordering, ("title", "id"))
        self.assertEqual(collection.get_icon_count(), 0)
        self.assertEqual(collection.get_count_of_active_icons(), 0)

    def test_custom_marker_icon(self):
        """
        Test the model of the MarkerIcon with the default settings.
        Тестирование модели MarkerIcon с настройками по умолчанию.
        """
        custom_marker_icon = self.custom_marker_icon_1
        title_icon_collection = self.title_icon_collection

        collection = IconCollection.objects.create(title=title_icon_collection)

        icon_marker = MarkerIcon.objects.create(
            icon_collection=collection,
            title=custom_marker_icon,
            svg="test_marker.svg")

        self.assertEqual(collection.icons.count(), 1)
        self.assertEqual(icon_marker.icon_collection.icons.count(), 1)

        self.assertFalse(MarkerIcon._meta.get_field('title').unique)
        self.assertFalse(MarkerIcon._meta.get_field('title').blank)
        self.assertEqual(MarkerIcon._meta.get_field(
            'title').max_length, 60)

        self.assertTrue(bool(icon_marker.svg))
        self.assertEqual(icon_marker.size_width, 46)
        self.assertEqual(icon_marker.size_height, 60)
        self.assertEqual(icon_marker.offset_x, Decimal("-23.0"))
        self.assertEqual(icon_marker.offset_y, Decimal(-60))
        self.assertTrue(icon_marker.active)
        self.assertEqual(icon_marker.__str__(), custom_marker_icon)
        self.assertEqual(MarkerIcon._meta.ordering, ("title", "id"))
        self.assertEqual(icon_marker.get_collection_name(),
                         title_icon_collection)

        self.assertEqual(icon_marker.get_collection_name(),
                         title_icon_collection)
        self.assertIsNone(icon_marker.clean())

        self.assertEqual(icon_marker.get_size(), "[46,60]")
        self.assertEqual(icon_marker.get_offset(), '[-23.0,-60]')

    def test_icon_collection_full(self):
        """
        Testing IconCollection model with Map and MarkerIcon models.
        Тестирование модели IconCollection с моделями Map и MarkerIcon.
        """
        title_icon_collection = self.title_icon_collection
        custom_marker_icon_1 = self.custom_marker_icon_1
        custom_marker_icon_2 = self.custom_marker_icon_2
        title_map = self.title_map

        icon_collection = IconCollection.objects.create(
            title=title_icon_collection)

        icon_marker_1 = MarkerIcon.objects.create(
            icon_collection=icon_collection,
            title=custom_marker_icon_1,
            svg="test_marker.svg")

        icon_marker_2 = MarkerIcon.objects.create(
            icon_collection=icon_collection,
            title=custom_marker_icon_2,
            svg="test_marker.svg")

        test_map = Map.objects.create(
            title=title_map, icon_collection=icon_collection)

        self.assertEqual(icon_collection.icons.count(), 2)
        self.assertEqual(icon_collection.map_icon_collection.count(), 1)

        self.assertEqual(icon_marker_1.icon_collection.title,
                         title_icon_collection)
        self.assertEqual(icon_marker_2.icon_collection.title,
                         title_icon_collection)
        self.assertEqual(test_map.icon_collection.title, title_icon_collection)

        self.assertEqual(icon_marker_1.get_size(), "[46,60]")
        self.assertEqual(icon_marker_1.get_offset(), '[-23.0,-60]')

        self.assertEqual(icon_marker_2.get_size(), "[46,60]")
        self.assertEqual(icon_marker_2.get_offset(), '[-23.0,-60]')

        self.assertEqual(
            icon_collection.map_icon_collection.all().first().title, title_map)
        self.assertEqual(icon_collection.icons.all()[
                         0].title, custom_marker_icon_1)
        self.assertEqual(icon_collection.icons.all()[
                         1].title, custom_marker_icon_2)

    def test_counter_id(self):
        """
        Testing CounterID model.
        Тестирование модели CounterID.
        """
        counter_id = CounterID.objects.create()

        self.assertFalse(CounterID._meta.get_field('num_id').blank)
        self.assertEqual(CounterID._meta.get_field('num_id').default, 1)
        self.assertEqual(counter_id.num_id, 1)

    def test_statistics(self):
        """
        Testing Statistics model.
        Тестирование модели Statistics.
        """
        obj_type = 'Point'
        obj_id = 1
        ip = '127.0.0.1'
        likes = True

        statistics = Statistics.objects.create(
            obj_type=obj_type,
            obj_id=obj_id,
            ip=ip,
            likes=likes
        )

        self.assertEqual(Statistics._meta.get_field(
            'obj_type').get_internal_type(), 'CharField')
        self.assertFalse(Statistics._meta.get_field(
            'obj_type').blank)
        self.assertEqual(Statistics._meta.get_field(
            'obj_type').default, "")
        self.assertEqual(statistics.obj_type, obj_type)

        self.assertEqual(Statistics._meta.get_field(
            'obj_id').get_internal_type(), 'PositiveIntegerField')
        self.assertFalse(Statistics._meta.get_field(
            'obj_id').blank)
        self.assertEqual(Statistics._meta.get_field(
            'obj_id').default, 0)
        self.assertEqual(statistics.obj_id, obj_id)

        self.assertEqual(Statistics._meta.get_field(
            'ip').get_internal_type(), 'GenericIPAddressField')
        self.assertFalse(Statistics._meta.get_field(
            'ip').blank)
        self.assertTrue(Statistics._meta.get_field(
            'ip').null)
        self.assertEqual(statistics.ip, ip)

        self.assertEqual(Statistics._meta.get_field(
            'likes').get_internal_type(), 'BooleanField')
        self.assertFalse(Statistics._meta.get_field(
            'likes').blank)
        self.assertFalse(Statistics._meta.get_field(
            'likes').default)
        self.assertEqual(statistics.likes, likes)

        self.assertEqual(Statistics._meta.get_field(
            'timestamp').get_internal_type(), 'DateTimeField')
        self.assertFalse(Statistics._meta.get_field(
            'timestamp').blank)
'''



# -*- coding: utf-8 -*-
import json
import re
from decimal import ROUND_CEILING, ROUND_HALF_UP, Decimal

from django.apps import apps
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist

from .utils import get_size_correction, get_size_from_svg


def icon_cluster_size_correction(instance, **kwargs):
    """Cluster Icons - Size correction and offset correction."""
    image = instance.svg
    size_width = instance.size_width
    size_height = instance.size_height

    if bool(image) and (size_width == 0 or size_height == 0):
        sizes_svg = get_size_from_svg(image)
        width = sizes_svg["width"]
        height = sizes_svg["height"]
        sizes = get_size_correction(width, height)
        width = sizes[0]
        height = sizes[1]

        instance.size_width = width
        instance.size_height = height

        instance.offset_x = ((Decimal(width) / Decimal(2))
                             .quantize(Decimal(".0"), ROUND_CEILING)) * Decimal(-1)
        instance.offset_y = ((Decimal(height) / Decimal(2))
                             .quantize(Decimal(".0"), ROUND_HALF_UP)) * Decimal(-1)
        instance.save()


def icon_marker_size_correction(instance, **kwargs):
    """Marker Icon - Size correction and offset correction."""
    image = instance.svg
    size_width = instance.size_width
    size_height = instance.size_height

    if bool(image) and (size_width == 0 or size_height == 0):
        sizes_svg = get_size_from_svg(image)
        width = sizes_svg["width"]
        height = sizes_svg["height"]
        sizes = get_size_correction(width, height)
        width = sizes[0]
        height = sizes[1]

        instance.size_width = width
        instance.size_height = height

        instance.offset_x = ((Decimal(width) / Decimal(2))
                             .quantize(Decimal(".0"), ROUND_CEILING))
        instance.offset_y = Decimal(height) * Decimal(-1)

        if instance.offset_x.to_integral_exact() - instance.offset_x == 0.5:
            instance.offset_x += Decimal("0.1")
            instance.offset_x *= Decimal(-1)
        else:
            instance.offset_x *= Decimal(-1)

        instance.save()


def refresh_json_code(instance, **kwargs):
    """Refresh JSON-code if change Subcategories."""
    instance.save()


def placemark_delete_statistics(instance, **kwargs):
    """Placemark - Delete orphaned statistics"""
    apps.get_model('djeym', 'Statistics').objects.filter(
        obj_type='Point', obj_id=instance.pk).delete()


def polyline_delete_statistics(instance, **kwargs):
    """Polyline - Delete orphaned statistics"""
    apps.get_model('djeym', 'Statistics').objects.filter(obj_type='LineString',
                                                         obj_id=instance.pk).delete()


def polygon_delete_statistics(instance, **kwargs):
    """Polygon - Delete orphaned statistics"""
    apps.get_model('djeym', 'Statistics').objects.filter(
        obj_type='Polygon', obj_id=instance.pk).delete()


def refresh_icon(instance, **kwargs):
    """Refresh icon (slug) in placemarks after refreshing icon in MarkerIcon."""
    maps = apps.get_model('djeym', 'Map').objects.filter(
        icon_collection=instance.icon_collection)
    for map in maps:
        placemarks = apps.get_model('djeym', 'Placemark').objects.filter(
            ymap=map, icon_slug=instance.slug)
        for placemark in placemarks:
            placemark.save()


# Save all json settings
def save_json_settings(json_settings, editor):
    """Save all json settings"""
    generalSettings = editor['generalSettings']
    mapControls = editor['mapControls']
    heatmapSettings = editor['heatmapSettings']
    categories = editor['categories']
    gradient = heatmapSettings['panel2']['gradient']
    heatmap = {
        'isActive': heatmapSettings['panel1']['isActive'],
        'radius': heatmapSettings['panel2']['radius'],
        'dissipating': heatmapSettings['panel2']['dissipating'],
        'opacity': heatmapSettings['panel2']['opacity'],
        'intensity': heatmapSettings['panel2']['intensity'],
        'gradient': {
            'color1': gradient['color1'],
            'color2': gradient['color2'],
            'color3': gradient['color3'],
            'color4': gradient['color4']
        },
    }
    front = {
        'themeType': generalSettings['themeType'],
        'colorControls': generalSettings['colorControlsTheme'],
        'colorButtonsText': generalSettings['colorButtonsTextTheme'],
        'openPanel': generalSettings['controls']['panel3_71'][2]['isActive'],
        'widthPanel': generalSettings['controls']['panel3_71'][1]['widthPanelFront'],
        'mapCenter': editor['ymap']['mapCenter'],
        'mapZoom': editor['ymap']['mapZoom'],
        'cluster': editor['ymap']['cluster'],
        'activeControls': mapControls['activeControls'],
        'isSearchByOrganization': mapControls['controls'][2]['isActive'],
        'isRoundTheme': generalSettings['controls']['panel1_69'][0]['isActive'],
        'isPanorama': generalSettings['controls']['panel1_69'][1]['isActive'],
        'currentTile': editor['tileSources']['currentTile'],
        'heatmap': heatmap,
        'colorBackgroundCountObjects': generalSettings['colorBackgroundCountObjects'],
        'textColorCountObjects': generalSettings['textColorCountObjects'],
        'isClusterize': generalSettings['controls']['panel3_71'][0]['isActive'],
        'isIconContentLayout': generalSettings['controls']['panel1_69'][3]['isActive'],
        'balloonContentLayout': generalSettings['controls']['panel1_69'][2]['layout'],
        'сategoryIcons': [icon for icon in categories['tabIcons'].values()],
        'multiple': categories['multiple'],
        'filters': categories['filters'],
        'imgBgPanel': generalSettings['controls']['panel3_71'][3]['large'],
        'tinting': generalSettings['tintingPanelFront'],
        'hideGeoTypes': categories['hideGeoTypes'],
        'hideGroupNames': categories['hideGroupNames'],
        'centerGeoTypes': categories['centerGeoTypes'],
        'geoTypeNameMarker': categories['geoTypeNameMarker'],
        'geoTypeNameRoute': categories['geoTypeNameRoute'],
        'geoTypeNameTerritory': categories['geoTypeNameTerritory'],
        'groupNameCategories': categories['groupNameCategories'],
        'groupNameSubcategories': categories['groupNameSubcategories'],
        'controlsShape': categories['controlsShape'],
        'effectRipple': categories['effectRipple']
    }
    json_settings.editor = json.dumps(editor, ensure_ascii=False)
    json_settings.front = json.dumps(front, ensure_ascii=False)
    json_settings.save()


# Tile Sources - Add random subdomains. Add api key or access token.
def random_domain(source, apikey):
    """Tile Sources - Add random subdomains. Add api key or access token."""
    source = re.sub('\r?\n', "", source)
    count_elem = re.search(r'\[\[(.+)\]\]', source)

    if count_elem is not None:
        count_elem = len(eval(count_elem.group(0))[0])

    value = re.sub(r'\[(\[.+\])\]',
                   '" + \\1[ Math.round( Math.random() * {} ) ] + "'
                   .format(int(count_elem) - 1), source)
    if len(apikey) > 0:
        var_key = re.search(r'\{\{(.+)\}\}', value).group(0)
        clean_var_key = re.sub(r'{{|}}', "", var_key)
        value = value.replace(var_key, clean_var_key + apikey, 1)
    else:
        value = re.sub(r'\{\{.+\}\}', "", value)
    return value


def convert_all_settings_to_json(instance, **kwargs):
    """Converting and updating all settings of Maps to JSON."""

    class_name = instance.__class__.__name__
    is_map = class_name == 'Map'

    if is_map:
        ymaps = [instance]
    else:
        ymaps = apps.get_model('djeym', 'Map').objects.all()
        if class_name == 'MarkerIcon' and \
                ymaps.filter(icon_collection=instance.icon_collection).count() == 0:
            return
        elif class_name == 'GeneralSettings' \
                or class_name == 'Preset' \
                or class_name == 'MapControls' \
                or class_name == 'HeatmapSettings':
            try:
                ymaps = [ymaps.get(pk=instance.ymap.pk)]
            except ObjectDoesNotExist:
                ymaps = []

    for ymap in ymaps:
        if not hasattr(ymap, 'json_settings'):
            return

        json_settings = ymap.json_settings
        editor = json.loads(json_settings.editor)

        # Get Icon Collection
        if class_name == 'MarkerIcon' or is_map:
            icon_collection = ymap.icon_collection.icons.filter(active=True)
            icon_collection = [{
                'title': item.title,
                'slug': item.slug,
                'url': item.svg.url,
                'size': json.loads(item.get_size()),
                'offset': json.loads(item.get_offset())
            } for item in icon_collection]
            editor['iconCollection'] = icon_collection
            if not is_map:
                save_json_settings(json_settings, editor)
                continue

        # Get categories
        if bool(re.search(r'Category', class_name)) or is_map:
            default_filters = {
                'a': [],  # Categories of placemarks
                'b': [],  # Subcategories of placemarks
                'c': [],  # Categories of Routes
                'd': [],  # Subcategories of Routes
                'e': [],  # Categories of Territories
                'f': []  # Subcategories of Territories
            }

            if editor.get('categories') is None:
                categories = {
                    'multiple': True,
                    'tabIcons': {
                        'marker': 'mdi-map-marker',
                        'route': 'mdi-routes',
                        'territory': 'mdi-beach'
                    },
                    'centerGeoTypes': False,
                    'hideGeoTypes': False,
                    'geoTypeNameMarker': "",
                    'geoTypeNameRoute': "",
                    'geoTypeNameTerritory': "",
                    'hideGroupNames': False,
                    'groupNameCategories': "",
                    'groupNameSubcategories': "",
                    'controlsShape': 'shaped',
                    'effectRipple': True,
                    'filters': default_filters
                }
            else:
                categories = editor['categories']
                categories['filters'] = default_filters

            raw_categories = {
                'a': ymap.categories_placemark.filter(active=True),
                'b': ymap.subcategories_placemark.filter(active=True),
                'c': ymap.categories_polyline.filter(active=True),
                'd': ymap.subcategories_polyline.filter(active=True),
                'e': ymap.categories_polygon.filter(active=True),
                'f': ymap.subcategories_polygon.filter(active=True)
            }
            multiple = categories['multiple']
            flag = True

            for key, val in raw_categories.items():
                if bool(val):
                    lock = True
                    for item in val:
                        categories['filters'][key].append({
                            'id': item.pk,
                            'title': item.title,
                            'icon': item.category_icon,
                            'color': item.category_color,
                            'isActive': lock and flag
                        })
                        lock = multiple
                flag = not flag
            editor['categories'] = categories

        # Get tile sources
        if class_name == 'TileSource' or is_map:
            tile_list = [{
                'id': 0,
                'img': '{}djeym/img/default_tile.png'.format(settings.STATIC_URL),
                'title': 'Default',
                'maxZoom': 23,
                'isActive': not bool(ymap.tile)
            }]
            current_tile_id = ymap.tile.pk if bool(ymap.tile) else 0
            tiles = apps.get_model('djeym', 'TileSource').objects.all()
            current_tile = tiles.filter(pk=current_tile_id).first()
            if current_tile is not None:
                current_tile = {
                    'title': current_tile.title,
                    'maxZoom': current_tile.maxzoom,
                    'minZoom': current_tile.minzoom,
                    'copyrights': current_tile.copyrights,
                    'randomTileUrl': '"{}"'.format(random_domain(
                        current_tile.source, current_tile.apikey))
                }
            tile_list.extend(
                [{
                    'id': item.pk,
                    'img': item.middle.url,
                    'title': item.title,
                    'maxZoom': item.maxzoom,
                    'isActive': item.pk == current_tile_id
                } for item in tiles]
            )
            tile_sources = {
                'tiles': tile_list,
                'currentTile': current_tile
            }
            editor['tileSources'] = tile_sources
            if not is_map:
                save_json_settings(json_settings, editor)
                continue

        # Get general settings
        if class_name == 'GeneralSettings' or is_map:
            general_settings = ymap.general_settings
            general_settings = {
                'controls': {
                    'panel1_69': [
                        {'icon': 'mdi-alpha-t-circle-outline',
                            'isActive': general_settings.roundtheme},
                        {'icon': 'mdi-airplay',
                            'isActive': general_settings.panorama},
                        {'icon': 'mdi-comment-text-outline',
                            'layout': general_settings.cluster_layout},
                        {
                            'icon': 'mdi-alpha-s-circle-outline',
                            'count': None,
                            'isActive': general_settings.cluster_icon_content
                        },
                        {'icon': 'mdi-theme-light-dark', 'theme': None}
                    ],
                    'panel2_70': [
                        {'icon': 'mdi-chart-bubble',
                            'isActive': general_settings.clustering_edit},
                        {'icon': 'mdi-arrow-split-vertical',
                            'widthPanelEditor': general_settings.width_panel_editor}
                    ],
                    'panel3_71': [
                        {'icon': 'mdi-chart-bubble',
                            'isActive': general_settings.clustering_site},
                        {'icon': 'mdi-arrow-split-vertical',
                            'widthPanelFront': general_settings.width_panel_front},
                        {'icon': 'mdi-arrow-expand-right',
                         'isActive': general_settings.open_panel_front},
                        {'imgBgPanelFront': general_settings.img_bg_panel_front_thumb.url if
                         bool(general_settings.img_bg_panel_front) else None,
                         'large': general_settings.img_bg_panel_front_large.url if
                         bool(general_settings.img_bg_panel_front) else None},
                        {'icon': 'mdi-arrow-split-vertical',
                            'widthMapFront': general_settings.width_map_front},
                        {'icon': 'mdi-arrow-split-horizontal',
                            'heightMapFront': general_settings.height_map_front},
                    ]
                },
                'themeType': general_settings.theme_type,
                'colorControlsTheme': general_settings.controls_color,
                'colorButtonsTextTheme': general_settings.buttons_text_color,
                'colorBackgroundCountObjects': general_settings.cluster_icon_content_bg_color,
                'textColorCountObjects': general_settings.cluster_icon_content_txt_color,
                'tintingPanelFront': general_settings.tinting_panel_front
            }
            editor['generalSettings'] = general_settings
            if not is_map:
                save_json_settings(json_settings, editor)
                continue

        # Get Map controls
        if class_name == 'MapControls' or is_map:
            map_controls = ymap.map_controls
            map_controls = {
                'controls': [
                    {'img': settings.STATIC_URL + 'djeym/img/map_controls/geolocation.png',
                     'width': 96, 'isActive': map_controls.geolocation},
                    {'img': settings.STATIC_URL + 'djeym/img/map_controls/search.png',
                     'width': 264, 'isActive': map_controls.search},
                    {'img': settings.STATIC_URL + 'djeym/img/map_controls/provider.png',
                     'width': 335, 'isActive': map_controls.provider},
                    {'img': settings.STATIC_URL + 'djeym/img/map_controls/route.png',
                     'width': 339, 'isActive': map_controls.route},
                    {'img': settings.STATIC_URL + 'djeym/img/map_controls/traffic.png',
                     'width': 289, 'isActive': map_controls.traffic},
                    {'img': settings.STATIC_URL + 'djeym/img/map_controls/typeselector.png',
                     'width': 273, 'isActive': map_controls.typeselector},
                    {'img': settings.STATIC_URL + 'djeym/img/map_controls/fullscreen.png',
                     'width': 96, 'isActive': map_controls.fullscreen},
                    {'img': settings.STATIC_URL + 'djeym/img/map_controls/zoom.png',
                     'width': 99, 'isActive': map_controls.zoom},
                    {'img': settings.STATIC_URL + 'djeym/img/map_controls/ruler.png',
                     'width': 102, 'isActive': map_controls.ruler}
                ],
                'activeControls': map_controls.get_active_control_list()
            }
            editor['mapControls'] = map_controls
            if not is_map:
                save_json_settings(json_settings, editor)
                continue

        # Get Heatmap
        if class_name == 'HeatmapSettings' or is_map:
            heatmap_settings = ymap.heatmap_settings
            heatmap_settings = {
                'panel1': {'isActive': heatmap_settings.active},
                'panel2': {
                    'gradient': {
                        'color1': heatmap_settings.gradient_color1,
                        'color2': heatmap_settings.gradient_color2,
                        'color3': heatmap_settings.gradient_color3,
                        'color4': heatmap_settings.gradient_color4
                    },
                    'radius': heatmap_settings.radius,
                    'dissipating': heatmap_settings.dissipating,
                    'opacity': heatmap_settings.opacity,
                    'intensity': heatmap_settings.intensity
                }
            }
            editor['heatmapSettings'] = heatmap_settings
            if not is_map:
                save_json_settings(json_settings, editor)
                continue

        # Get load indicators
        if class_name == 'LoadIndicator' or is_map:
            load_indicators = apps.get_model(
                'djeym', 'LoadIndicator').objects.all()
            load_indicators = {
                'indicators': [{
                    'title': item.title,
                    'slug': item.slug,
                    'img': item.svg.url
                } for item in load_indicators],
                'size': ymap.load_indicator_size,
                'speed': ymap.animation_speed,
                'disableAnimation': ymap.disable_indicator_animation,
                'currentIndicator': ymap.load_indicator
                .slug if ymap.load_indicator is not None else ""
            }
            editor['loadIndicators'] = load_indicators
            if not is_map:
                save_json_settings(json_settings, editor)
                continue

        # Get Presets
        if class_name == 'Preset' or is_map:
            presets = ymap.presets.all()
            presets = [{
                'id': item.pk,
                'title': item.title,
                'icon': item.icon,
                'description': item.description,
                'autoheader': item.autoheader,
                'autobody': item.autobody,
                'autofooter': item.autofooter,
                'placemark': item.placemark,
                'polyline': item.polyline,
                'polygon': item.polygon,
                'position': item.position
            } for item in presets]
            editor['presets'] = presets
            if not is_map:
                save_json_settings(json_settings, editor)
                continue

        # Get settings for YMap and Cluster
        cluster = ymap.icon_cluster
        tmp_ymap = {
            'mapCenter': json.loads('[{0}, {1}]'.format(ymap.latitude, ymap.longitude)),
            'mapZoom': ymap.zoom,
            'cluster': {}
        }
        if cluster is not None:
            tmp_ymap['cluster'] = {
                'url': cluster.svg.url,
                'size': [cluster.size_width, cluster.size_height],
                'offset': json.loads('[{0:f}, {1:f}]'.format(
                    cluster.offset_x, cluster.offset_y))
            }
        editor['ymap'] = tmp_ymap

        # Save all json settings
        save_json_settings(json_settings, editor)
